/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#ifndef BLUETOOTHHANDLER_H__
#define BLUETOOTHHANDLER_H__

#include <es_sock.h>
#include <bt_sock.h>
#include <btextnotifiers.h>
#include <btmanclient.h>
#include <btsdp.h>
#include <eikprogi.h>

#ifdef BEMUSED_SERIES_60
  #include <AknWaitDialog.h>
  #include <AknProgressDialog.h>
#else
  #include "BemusedProgressDialog.h"
#endif

#include "DirTreeNode.h"

#ifdef BEMUSED_SERIES_60
  _LIT(KDownloadedFilesPath, "c:\\nokia\\sounds\\digital");
#else
  _LIT(KDownloadedFilesPath, "c:\\documents\\Media files\\audio\\unfiled");
  _LIT(KDownloadedFilesPathMs, "d:\\Media files\\audio\\unfiled");
#endif

const TInt KBemusedPort = 11;

const TInt KSerialPortService = 0x1101;

const TInt KNormalCommandTimeout = 20 * 1000000;
const TInt KLongCommandTimeout = 300 * 1000000;

enum TBemusedCommand {
	EBemusedNone = 0,
	EBemusedConnect,
	EBemusedList,
	EBemusedCurrentTrackInfo,
	EBemusedPlayFile,
	EBemusedAddFileToPlaylist,
	EBemusedFileInfo,
	EBemusedDownloadFile,
	EBemusedSetVolume,
	EBemusedPlay,
	EBemusedStop,
	EBemusedPause,
	EBemusedFastForward,
	EBemusedRewind,
	EBemusedNextTrack,
	EBemusedPreviousTrack,
	EBemusedShuffle,
	EBemusedRepeat,
	EBemusedShutDown,
	EBemusedFindServer,
	EBemusedGetPlaylist,
	EBemusedSelectInPlaylist,
	EBemusedRemoveFromPlaylist,
	EBemusedRemoveAllFromPlaylist,
	EBemusedDetailedTrackInfo,
	EBemusedFullScreen,
	EBemusedGetVersion,
	EBemusedGetVolume
	};

enum TActionOnStopping
	{
	EStopImmediately,
	EFadeOut,
	EStopAtEndOfSong
	};

class TBemusedSettings;

class MBluetoothListener
	{
public:
	virtual void ConnectComplete(TInt aError, const TBTDevAddr& aAddress, TInt aPort) = 0;
	virtual void CommandComplete(TBemusedCommand aCommand, TInt aError) = 0;
	virtual void RefreshFileListL(CDirTreeNode* aDirectory) = 0;
	virtual void SetInfo(TBool aPlaying, TBool aPaused, TInt aSongLength, TInt aCurrentTime,
		TBool aShuffle, TBool aRepeat, const TDesC8& aSongNameGuess) = 0;
	virtual void SetDetailedInfo(TInt aSampleRate, TInt aBitRate, TInt aChannels) = 0;
	virtual void ReceivedVolume(TInt aVolume) = 0;
	virtual TBemusedSettings& Settings() = 0;
	virtual void SetPlaylistIndex(TInt aIndex) = 0;
	virtual RPointerArray<HBufC8>& Playlist() = 0;
	virtual CCoeEnv* CoeEnv() = 0;
	};

class CBluetoothHandler : public CActive, public MSdpAgentNotifier, public MSdpAttributeValueVisitor
	, public MProgressDialogCallback
	{
public:
	enum TBluetoothState {
		EIdle,
		EReadingSongListFile,
		ESelectingServer,
		EInquiringService,
		EConnecting,
		EReading,
		EWriting
		};

	static CBluetoothHandler* NewL(MBluetoothListener* aListener,
		const TBTDevAddr& aAutoConnectAddress, TInt aAutoConnectPort);
	~CBluetoothHandler();

	void ReadSongListFile(const TDesC& aFileName);
	void Connect();
	
public: // from MSdpAgentNotifier
	virtual void AttributeRequestComplete(TSdpServRecordHandle aHandle, TInt aError);
	virtual void AttributeRequestResult(TSdpServRecordHandle aHandle, TSdpAttributeID aAttrID,
		CSdpAttrValue* aAttrValue);
	virtual void NextRecordRequestComplete(TInt aError, TSdpServRecordHandle aHandle,
		TInt aTotalRecordsCount);

public: // from MSdpAttributeValueVisitor
	virtual void VisitAttributeValueL(CSdpAttrValue &aValue, TSdpElementType aType);
    virtual void StartListL(CSdpAttrValueList &aList);
    virtual void EndListL();

public: // from MProgressDialogCallback.
    virtual void DialogDismissedL(TInt aButtonId);

public: // New functions.
	void SortDirTree(CDirTreeNode* aDirectory);

	void RequestList(TBool aGetEntireTree);
	void RequestDirectoryList(CDirTreeNode* aNode);
	void RequestInfo();
	void RequestPlayFile(const TDesC8& aFileName);
	void RequestAddFileToPlaylist(const TDesC8& aFileName);
	void RequestFileInfo(const TDesC8& aFileName);
	void RequestDownloadFile(const TDesC8& aFileName);
	void RequestSetVolume(TUint8 aVolume);
	void RequestPlay();
	void RequestStop(TActionOnStopping aAction);
	void RequestPause();
	void RequestNextTrack();
	void RequestPreviousTrack();
	void RequestFastForward();
	void RequestRewind();
	void RequestShuffle(TBool aShuffleOn);
	void RequestRepeat(TBool aRepeatOn);
	void RequestShutDown();
	void RequestFindServer();
	void RequestGetPlaylist();
	void RequestSelectInPlaylist(TInt aIndex);
	void RequestRemoveAllFromPlaylist();
	void RequestDetailedTrackInfo();
	void RequestFullScreen();

	TPtrC8 ReadListFileName(const TDesC8& aPacket, TInt& aStartPos);

	static TInt CompareNodes(const CDirTreeNode& aNode1, const CDirTreeNode& aNode2);

private: // Inherited from CActive.
	void RunL();
	void DoCancel();
	TInt RunError(TInt aError);

private: // New functions.
	CBluetoothHandler(MBluetoothListener* aListener,
		const TBTDevAddr& aAutoConnectAddress,
		TInt aAutoConnectPort);
	void ConstructL();

	void CloseAllResources();

	void ParseCommandResponse();
	TBool CommandExpectsResponse(TBemusedCommand aCommand);
	void WriteCommand();

	void HandleReadFileCompleteL();
	void HandleSelectServerCompleteL();
	void HandleConnectionCompleteL();
	void HandleReadCompleteL();
	void HandleWriteCompleteL();

	void ParseListResponse();
	void ParseCurrentTrackInfoResponse();
	void ParseFileInfoResponse();
	void ParseDownloadFileResponse();
	void ParseFindServerResponse();
	void ParseGetPlaylistResponseL();
	void ParseDetailedTrackInfoResponse();
	void ParseGetVersionResponse();
	void ParseGetVolumeResponse();

	TInt ParseListPacketL(TDes8& aPacket);
	TInt ParsePlaylistPacketL(TDes8& aPacket);

	void ConnectionFailed(TInt aError);

	static TInt CommandTimerCallBack(TAny* aBluetoothHandler);
	void CommandTimedOut();
	void FindServerNextChannel();

	void ShowWaitDialogL(TInt aResourceId);

private:
	RSocketServ iSocketServ;
	RSocket iSocket;

	TBluetoothState iState;
	RNotifier iNotifier;

	TBTDeviceResponseParamsPckg iDeviceSelectionPckg;
	TBTDeviceResponseParamsPckg iDeviceFoundPckg;

	TSockXfrLength iXfrLength;
	TBuf8<1024> iPacketBuf;
	TBuf8<2048> iTotalPacketBuf;

	CSdpAgent* iSdpAgent;
	CSdpSearchPattern* iSdpSearchPattern;
	TInt iSdpCurrentRecordCount;
	TInt iSdpTotalRecordCount;

#ifdef BEMUSED_SERIES_60
	CAknWaitDialog* iWaitDialog;
	CAknProgressDialog* iProgressDialog;
#else
	CBemusedProgressDialog* iProgressDialog;
#endif

	CEikProgressInfo* iProgressInfo;

	TUUID iServiceClass;
	TUint iRemotePort;
	
	TBool iConnected;

	enum TVersionState
		{
		ENotGotVersion,
		EGettingVersion,
		EGotVersion,
		EGettingVolume,
		EGotVolume
		};

	TVersionState iVersionState;
	TInt iServerMajorVersion;
	TInt iServerMinorVersion;
	TBuf8<1024> iVersionQueuedPacketBuf;

	TBemusedCommand iCurrentCommand;
	TBemusedCommand iQueuedCommand;
	TBool iWaitingForResponseData;

	RFile iSongListFile;

	RFile iDownloadFile;
	TFileName iDownloadFileName;
	TInt iDownloadFileLength;
	TInt iDownloadFileReceived;

	MBluetoothListener* iListener;
	
	CDirTreeNode* iDirTreeRoot;
	CDirTreeNode* iDirTreeCurrentNode;
	CDirTreeNode* iDirTreeNodeToExpand;

	TBTDevAddr iAutoConnectAddress;
	TInt iAutoConnectPort;

	TInt iFindServerPort;
	CPeriodic* iCommandTimer;

	TInt iPlaylistPos;
	
	TNameEntry iNameEntry;
	RHostResolver iResolver;
	};

#endif