/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#include <eiktxlbm.h>
#include <eiklbv.h>

#include "BemusedBrowseContainer.h"
#include "BemusedApp.h"
#include "BemusedAppUi.h"

#ifdef BEMUSED_SERIES_60
  #include <AknQueryDialog.h>
  #include <AknUtils.h>

  #include "BemusedControlView.h"
  #include <Bemused.rsg>
#else
  #include <Bemused.rsg>
  #include "BemusedFindSongDialog.h"
#endif

#include <gdi.h>

void CBemusedBrowseContainer::ConstructL(CBemusedAppUi* aAppUi, const TRect& aRect)
    {
	iAppUi = aAppUi;

    CreateWindowL();

	iListItems = new (ELeave) CDesC16ArrayFlat(4);

	// Create our custom list box, specifying the font which the user's chosen.

	const CFont* font;

	switch (iAppUi->Settings().iSongBrowserFont)
		{
	case TBemusedSettings::EHuge:
		font = CEikonEnv::Static()->TitleFont();
		break;
#ifdef BEMUSED_SERIES_60
	case TBemusedSettings::ELarge:
		font = CEikonEnv::Static()->NormalFont();
		break;
	case TBemusedSettings::EMedium:
		font = CEikonEnv::Static()->LegendFont();
		break;
#else
	case TBemusedSettings::ELarge:
		font = CEikonEnv::Static()->LegendFont();
		break;
	case TBemusedSettings::EMedium:
		font = CEikonEnv::Static()->NormalFont();
		break;
#endif
	case TBemusedSettings::ESmall:
	default:
		font = CEikonEnv::Static()->DenseFont();
		break;
		}

	iListBox = new (ELeave) CBemusedListBox(font);

#ifdef BEMUSED_SERIES_60
	iListBox->ConstructL(this, EAknListBoxLoopScrolling | CEikListBox::EPaintedSelection);
#else
	iListBox->ConstructL(this, CEikListBox::EPaintedSelection);

	iListBox->SetListBoxObserver(this);
#endif

	CBemusedListItemDrawer* drawer = (CBemusedListItemDrawer*) iListBox->View()->ItemDrawer();

	drawer->SetColours(iTextForegroundColour, iTextBackgroundColour, iHighlightForegroundColour,
		iHighlightBackgroundColour);

	iListBox->Model()->SetOwnershipType(ELbmDoesNotOwnItemArray);
	iListBox->Model()->SetItemTextArray(iListItems);

	// Set the colour to be used in areas where there aren't any items in the list.
	iListBox->View()->SetBackColor(iTextBackgroundColour);
	iListBox->View()->ItemDrawer()->SetBackColor(iTextBackgroundColour);

	// Set the item height to a sensible value that can be used for all fonts.
	iListBox->SetItemHeightL(KItemHeight);

	// Create the scrolling indicators at the bottom of the screen.
	iScrollBarFrame = iListBox->CreateScrollBarFrameL(ETrue);
    iListBox->ScrollBarFrame()->SetScrollBarVisibilityL(
            CEikScrollBarFrame::EAuto, CEikScrollBarFrame::EAuto);

    SetRect(aRect);

	RefreshFileListL(iAppUi->CurrentDirectory());

	iListBox->Model()->SetItemTextArray(iListItems);
	iListBox->HandleItemAdditionL();
	iListBox->SetCurrentItemIndex(0);

    ActivateL();
    }

CBemusedBrowseContainer::~CBemusedBrowseContainer()
    {
	delete iListBox;
	delete iListItems;
    }

void CBemusedBrowseContainer::SizeChanged()
    {
	iListBox->SetRect(Rect());
	}

TInt CBemusedBrowseContainer::CountComponentControls() const
    {
	if (iListBox)
		return 1;
	else
		return 0;
    }

CCoeControl* CBemusedBrowseContainer::ComponentControl(TInt aIndex) const
    {
    switch (aIndex)
        {
        case 0:
			return iListBox;
        default:
            return NULL;
        }
    }

void CBemusedBrowseContainer::Draw(const TRect& /*aRect*/) const
    {
	// Nothing to be done, as the list box takes up the entire view.
	}

void CBemusedBrowseContainer::HandleControlEventL(CCoeControl* /*aControl*/,
												  TCoeEvent /*aEventType*/)
    {
    }

TKeyResponse CBemusedBrowseContainer::OfferKeyEventL(const TKeyEvent& aKeyEvent,
													 TEventCode aType)
    {
	if ((aKeyEvent.iCode == EKeyUpArrow) || (aKeyEvent.iCode == EKeyDownArrow))
		{
		((CBemusedListItemDrawer*) iListBox->View()->ItemDrawer())->ResetScrolling();
		return iListBox->OfferKeyEventL(aKeyEvent, aType);
		}
	else if ((aKeyEvent.iScanCode == '0') && (aType == EEventKey))
		{
		((CBemusedListItemDrawer*) iListBox->View()->ItemDrawer())->ResetScrolling();

		TInt currentIndex = iListBox->CurrentItemIndex();

		TInt itemsOnScreen = 6;
		if (iAppUi->Settings().iDisplayType == TBemusedSettings::EFullScreen)
			itemsOnScreen = 8;

		if (iListBox->Model()->ItemTextArray()->MdcaCount() > (currentIndex + itemsOnScreen))
			{
			iListBox->SetCurrentItemIndex(currentIndex + itemsOnScreen);
			}
		else
			{
			iListBox->SetCurrentItemIndex(iListBox->Model()->ItemTextArray()->MdcaCount() - 1);
			}
		iListBox->DrawNow();
		return EKeyWasConsumed;
		}
	else if ((aKeyEvent.iScanCode == '2') && (aType == EEventKey))
		{
		((CBemusedListItemDrawer*) iListBox->View()->ItemDrawer())->ResetScrolling();

		TInt currentIndex = iListBox->CurrentItemIndex();

		TInt itemsOnScreen = 6;
		if (iAppUi->Settings().iDisplayType == TBemusedSettings::EFullScreen)
			itemsOnScreen = 8;

		if (currentIndex > itemsOnScreen)
			{
			iListBox->SetCurrentItemIndex(currentIndex - itemsOnScreen);
			}
		else
			{
			iListBox->SetCurrentItemIndex(0);
			}
		iListBox->DrawNow();
		return EKeyWasConsumed;
		}

	return EKeyWasNotConsumed;
	}

void CBemusedBrowseContainer::RefreshFileListL(CDirTreeNode* aDirectory)
	{
	((CBemusedListItemDrawer*) iListBox->View()->ItemDrawer())->ResetScrolling();

	iAppUi->SetCurrentDirectory(aDirectory);
	iListItems->Reset();

	TBuf<256> resourceBuf;
	iAppUi->CoeEnv()->ReadResource(resourceBuf, R_UP_ITEM);

	if (aDirectory)
		{
		if (aDirectory->iParent)
			iListItems->AppendL(resourceBuf);

		CDirTreeNode* node = aDirectory->iFirstChild;
		TBuf<256> displayBuf;
		TMusicFileType fileType;

		while (node)
			{
			if (!FileShouldBeDisplayed(node))
				{
				node = node->iNextSibling;
				continue;
				}

			displayBuf.Copy(*node->iFileName);

			fileType = GetFileType(displayBuf);

			// Append a backslash for directories. Otherwise, do some checking based on the
			// file type.
			if ((node->iFileType == KFileTypeDir) || (node->iFileType == KFileTypeUnexpandedDir))
				{
				displayBuf.Append(_L("\\"));
				}
			else
				{
				// Hide file extension for ".mp3" files.
				if (fileType == EMp3File)
					displayBuf.Delete(displayBuf.Length() - 4, 4);
				}

			iListItems->AppendL(displayBuf);

			node = node->iNextSibling;
			}
		iListBox->Model()->SetItemTextArray(iListItems);

#ifndef BEMUSED_SERIES_60
		iListBox->ScrollToMakeItemVisible(0);
#endif

		iListBox->HandleItemAdditionL();
		iListBox->SetCurrentItemIndex(0);
		}
	else
		{
		iListBox->Model()->SetItemTextArray(iListItems);
#ifdef BEMUSED_SERIES_60
		iListBox->View()->DrawEmptyList(iListBox->Rect());
#endif
		iListBox->DrawNow();
		}
	}

void CBemusedBrowseContainer::FileSelectedL()
	{
	// do we want to go up a level?
	if ((iListBox->CurrentItemIndex() == 0) && (iAppUi->CurrentDirectory()->iParent))
		{
		UpOneLevelL();
		return;
		}

	CDirTreeNode* node = FindSelectedItem();
	if (!node)
		return;

	// do we want to expand a directory?
	if (node->iFileType == KFileTypeDir)
		{
		RefreshFileListL(node);
		iListBox->ScrollToMakeItemVisible(0);
		DrawNow();
		return;
		}

	// Do we need to send a command to the PC to refresh this list?
	if (node->iFileType == KFileTypeUnexpandedDir)
		{
		iAppUi->GetDirectoryList(node);
		return;
		}

	// if we get here, we want to play a file.

	TBuf8<1024> fullName;
	GetFullNodeFileName(node, fullName);

	TBool canPlay = EFalse;

	// This logic is used to decide whether, in the case where we're playing the first song
	// then adding subsequent songs, we want to play or add the selected song. The logic is:
	//   - if we haven't got any songs in the playlist at all, we want to play
	//   - if we have more than one song, and we're on the last song, and it's stopped,
	//     we want to play
	//   - if we have more than one song, we want to add

	if ((iAppUi->Playlist().Count() == 0) ||
		((iAppUi->PlaylistIndex() == (iAppUi->Playlist().Count()-1)) &&
		((iAppUi->SongState() == ENoSongLoaded) || (iAppUi->SongState() == ESongStopped))))
		{
		canPlay = ETrue;
		}

	if ((iAppUi->Settings().iSelectingItem == TBemusedSettings::EPlaysIt)  ||
		((iAppUi->Settings().iSelectingItem == TBemusedSettings::EPlaysThenAdds) &&
		(canPlay)))
		{
		iAppUi->PlayFile(node, fullName, EFalse);
		}
	else
		{
		iAppUi->PlayFile(node, fullName, ETrue);
		}
	}

void CBemusedBrowseContainer::PlaySelectedFileL(TBool aAddToPlaylist)
	{
	CDirTreeNode* node = FindSelectedItem();
	if (!node)
		return;

	TBuf8<1024> fullName;
	GetFullNodeFileName(node, fullName);

	iAppUi->PlayFile(node, fullName, aAddToPlaylist);
	}

void CBemusedBrowseContainer::DownloadSelectedFileL()
	{
	CDirTreeNode* node = FindSelectedItem();
	if (!node)
		return;

	TBuf8<1024> fullName;
	GetFullNodeFileName(node, fullName);

	iAppUi->DownloadFile(fullName);
	}

CDirTreeNode* CBemusedBrowseContainer::FindSelectedItem()
	{
	TInt index = iListBox->CurrentItemIndex();
	
	// If there isn't a currently selected item, give up.
	if ((index < 0) || (index >= iListItems->Count()))
		return NULL;

	// Skip the [up].
	if (iAppUi->CurrentDirectory()->iParent)
		index--;

	CDirTreeNode* node = iAppUi->CurrentDirectory()->iFirstChild;

	// Find the node the user selected.
	for (TInt count = 0; count < index; count++)
		{
		while (!FileShouldBeDisplayed(node))
			node = node->iNextSibling;

		if (node && node->iNextSibling)
			node = node->iNextSibling;
		else
			return NULL;
		}

	while (!FileShouldBeDisplayed(node))
		node = node->iNextSibling;

	return node;
	}

void CBemusedBrowseContainer::GetFullNodeFileName(CDirTreeNode* aNode, TDes8& aFileName)
	{
	aFileName.Zero();

	if (!aNode)
		return;

	aFileName.Copy(*aNode->iFileName);

	if (!aNode->iParent)
		return;

	do {
		aNode = aNode->iParent;

		// Allow for the case where the root has no filename.
		if (*aNode->iFileName != KNullDesC8)
			{
			aFileName.Insert(0, _L8("\\"));
			aFileName.Insert(0, *aNode->iFileName);
			}
		} while (aNode->iParent);
	}

void CBemusedBrowseContainer::UpOneLevelL()
	{
	if (iAppUi->CurrentDirectory() && iAppUi->CurrentDirectory()->iParent)
		{
		CDirTreeNode* childDirectory = iAppUi->CurrentDirectory();

		RefreshFileListL(iAppUi->CurrentDirectory()->iParent);
		((CBemusedListItemDrawer*) iListBox->View()->ItemDrawer())->ResetScrolling();

		// find the child directory in the parent directory.
		CDirTreeNode* node = iAppUi->CurrentDirectory()->iFirstChild;
		TInt count = 0;

		while (node)
			{
			if (node == childDirectory)
				{
				// allow for the [up] if it's there.
				if (iAppUi->CurrentDirectory()->iParent)
					iListBox->SetCurrentItemIndex(count + 1);
				else
					iListBox->SetCurrentItemIndex(count);
				break;
				}

			// We might need to do some special checking if the user has selected to only
			// display certain types of files. However, at the moment directories are always
			// listed before files, so this is unnecessary.

			node = node->iNextSibling;
			count++;
			}

		iListBox->ScrollToMakeItemVisible(iListBox->CurrentItemIndex());

		DrawNow();
		}
	}

TMusicFileType CBemusedBrowseContainer::GetFileType(const TDesC& aFileName)
	{
	TMusicFileType fileType = EUnknownFile;

	if ((aFileName.Length() > 4) && (!aFileName.Right(4).CompareF(_L(".mp3"))))
		fileType = EMp3File;
	else if ((aFileName.Length() > 4) && (!aFileName.Right(4).CompareF(_L(".m3u"))))
		fileType = EMp3File;
	else if ((aFileName.Length() > 4) && (!aFileName.Right(4).CompareF(_L(".mp2"))))
		fileType = EMusicFile;
	else if ((aFileName.Length() > 4) && (!aFileName.Right(4).CompareF(_L(".wma"))))
		fileType = EMusicFile;
	else if ((aFileName.Length() > 4) && (!aFileName.Right(4).CompareF(_L(".wav"))))
		fileType = EMusicFile;
	else if ((aFileName.Length() > 4) && (!aFileName.Right(4).CompareF(_L(".mid"))))
		fileType = EMusicFile;
	else if ((aFileName.Length() > 4) && (!aFileName.Right(4).CompareF(_L(".mod"))))
		fileType = EMusicFile;
	else if ((aFileName.Length() > 4) && (!aFileName.Right(4).CompareF(_L(".cda"))))
		fileType = EMusicFile;
	else if ((aFileName.Length() > 4) && (!aFileName.Right(4).CompareF(_L(".ogg"))))
		fileType = EMusicFile;

	return fileType;
	}

void CBemusedBrowseContainer::SetListEmptyStringL(const TDesC& aEmptyString)
	{
	if (iListBox)
		{
#ifdef BEMUSED_SERIES_60
		iListBox->View()->SetListEmptyTextL(aEmptyString);
#endif
		DrawNow();
		}
	}

void CBemusedBrowseContainer::SetColours(const TRgb& aTextForeground, const TRgb& aTextBackground,
								   const TRgb& aHighlightForeground, const TRgb& aHighlightBackground)
	{
	iTextForegroundColour = aTextForeground;
	iTextBackgroundColour = aTextBackground;
	iHighlightForegroundColour = aHighlightForeground;
	iHighlightBackgroundColour = aHighlightBackground;

	if (iListBox)
		{
		CBemusedListItemDrawer* drawer =
			(CBemusedListItemDrawer*) iListBox->View()->ItemDrawer();

		drawer->SetColours(iTextForegroundColour, iTextBackgroundColour,
			iHighlightForegroundColour, iHighlightBackgroundColour);

		iListBox->View()->SetBackColor(iTextBackgroundColour);
		}
	}

void CBemusedBrowseContainer::ScrollSelectedSong()
	{
	TInt index = iListBox->CurrentItemIndex();
	if ((index < 0) || (index >= iListBox->Model()->ItemTextArray()->MdcaCount()))
		return;

	if (((CBemusedListItemDrawer*) iListBox->View()->ItemDrawer())->ScrollSong(index))
		DrawNow();
	}

CDirTreeNode* CBemusedBrowseContainer::RecursiveFind(CDirTreeNode* aNode, TDes8& aName)
	{
	while (aNode)
		{
		if (!FileShouldBeDisplayed(aNode))
			{
			aNode = aNode->iNextSibling;
			continue;
			}

		if (aNode->iFileName->FindF(aName) != KErrNotFound)
			{
			iResultsFound++;
			if (iResultsFound > iFindResultsToSkip)
				return aNode;
			}

		if (aNode->iFirstChild)
			{
			CDirTreeNode* foundNode = RecursiveFind(aNode->iFirstChild, aName);

			if (foundNode)
				{
				iResultsFound++;
				if (iResultsFound > iFindResultsToSkip)
					return foundNode;
				}
			}

		aNode = aNode->iNextSibling;
		}

	return NULL;
	}

void CBemusedBrowseContainer::FindSongL(TBool aFindNext)
	{
	TFileName lastSearchedSong = iLastSearchedSong;

#ifdef BEMUSED_SERIES_60
	CAknTextQueryDialog* dialog = NULL;
#else
	CBemusedFindSongDialog* dialog = NULL;
#endif

	// Create a dialog to ask the user what to find, if we don't want
	// to just find the next occurrence.
	if (aFindNext)
		{
		if (iLastSearchedSong == KNullDesC)
			return;
		}
	else
		{
#ifdef BEMUSED_SERIES_60
		dialog = CAknTextQueryDialog::NewL(iLastSearchedSong);
		dialog->SetPredictiveTextInputPermitted(ETrue);
#else
		dialog = new (ELeave) CBemusedFindSongDialog(iLastSearchedSong);
#endif

		CleanupStack::PushL(dialog);
		}

	iResultsFound = 0;

	if (aFindNext || dialog->ExecuteLD(R_FIND_DIALOG))
		{
		// If the user wants to search for something else, don't skip
		// any of the results.
		if (iLastSearchedSong != lastSearchedSong)
			iFindResultsToSkip = 0;

		TBuf8<256> songToFind;
		songToFind.Copy(iLastSearchedSong);

		// Go back to the root directory from where we are now.
		CDirTreeNode* currentNode = iAppUi->CurrentDirectory();
		while (currentNode && currentNode->iParent)
			currentNode = currentNode->iParent;

		// Actually try to find the node.
		CDirTreeNode* foundNode = RecursiveFind(currentNode, songToFind);

		if (foundNode)
			{
			// We've found the node. Now we actually need to select it in the directory.

			RefreshFileListL(foundNode->iParent);

			if (iListBox->Model()->ItemTextArray()->MdcaCount() <= 1)
				{
				iListBox->DrawNow();
				iFindResultsToSkip++;
				if (!aFindNext)
					CleanupStack::Pop(); // dialog
				return;
				}

			TInt nodeIndex = 0;

			// Allow for the [up].
			if (foundNode->iParent->iParent)
				nodeIndex++;

			currentNode = foundNode->iParent->iFirstChild;
			while (currentNode && (currentNode != foundNode))
				{
				if (FileShouldBeDisplayed(currentNode))
					nodeIndex++;

				currentNode = currentNode->iNextSibling;
				}

			if (nodeIndex < iListBox->Model()->ItemTextArray()->MdcaCount())
				{
				((CBemusedListItemDrawer*) iListBox->View()->ItemDrawer())->ResetScrolling();
				iListBox->SetCurrentItemIndex(nodeIndex);
				iListBox->ScrollToMakeItemVisible(nodeIndex);
				iListBox->DrawNow();
				}
			else
				{
				iListBox->ScrollToMakeItemVisible(0);
				}

			// If the user searches again, we'll skip this result.
			iFindResultsToSkip++;
			}
		else
			{
			TBuf<256> resourceBuf;
			iAppUi->CoeEnv()->ReadResource(resourceBuf, R_FIND_FAILURE);

			lastSearchedSong.Format(resourceBuf, &iLastSearchedSong);
			CEikonEnv::Static()->InfoWinL(lastSearchedSong, KNullDesC);
			iFindResultsToSkip = 0;
			}
		}

	if (!aFindNext)
		CleanupStack::Pop(); // dialog
	}

TBool CBemusedBrowseContainer::FileShouldBeDisplayed(const CDirTreeNode* aNode)
	{
	if (!aNode)
		return EFalse;

	if (aNode->iFileType != KFileTypeFile)
		return ETrue;

	TBuf<256> fileName;

	fileName.Copy(*aNode->iFileName);
	TMusicFileType fileType = GetFileType(fileName);

	// Are we only supposed to display MP3s?
	if (iAppUi->Settings().iDisplayFiles == TBemusedSettings::EDisplayMp3s)
		{
		if (fileType == EMp3File)
			return ETrue;
		}
	// Are we only supposed to display music files?
	else if (iAppUi->Settings().iDisplayFiles == TBemusedSettings::EDisplayMusic)
		{
		if (fileType != EUnknownFile)
			return ETrue;
		}
	else
		{
		return ETrue;
		}

	return EFalse;
	}

////////////////////////////////////////////////////////////////////////////////
// UIQ-only code below this point.
////////////////////////////////////////////////////////////////////////////////

#ifndef BEMUSED_SERIES_60

void CBemusedBrowseContainer::HandleListBoxEventL(CEikListBox* aListBox, TListBoxEvent aEventType)
	{
	if (aEventType == EEventItemDoubleClicked)
		FileSelectedL();
	}

void CBemusedBrowseContainer::SelectNextFile()
	{
	TInt currentIndex = iListBox->CurrentItemIndex();

	if (currentIndex < (iListItems->Count()-1))
		{
		iListBox->SetCurrentItemIndex(currentIndex + 1);
		iListBox->ScrollToMakeItemVisible(currentIndex + 1);
		iListBox->DrawDeferred();
		}
	}

void CBemusedBrowseContainer::SelectPreviousFile()
	{
	TInt currentIndex = iListBox->CurrentItemIndex();

	if (currentIndex > 0)
		{
		iListBox->SetCurrentItemIndex(currentIndex - 1);
		iListBox->ScrollToMakeItemVisible(currentIndex - 1);
		iListBox->DrawDeferred();
		}
	}

#endif // !BEMUSED_SERIES_60

// End of File  
