/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#include "BemusedControlContainer.h"

#ifdef BEMUSED_SERIES_60
  #include "BemusedControlView.h"
#else
  #include "BemusedMainView.h"
  #include "Bemused.hrh"
#endif

#include <eikenv.h>

#ifdef BEMUSED_SERIES_60
  void CBemusedControlContainer::ConstructL(CBemusedAppUi* aAppUi, CBemusedControlView* aView,
#else
  void CBemusedControlContainer::ConstructL(CBemusedAppUi* aAppUi, CBemusedMainView* aView,
#endif
										  const TRect& aRect)
    {
	iAppUi = aAppUi;
	iView = aView;

	// Use off-screen bitmaps for the track name and track time. As these change
	// every second, it's important to avoid flicker. To save memory, off-screen
	// bitmaps are currently not used for any other display components.

	iTrackNameBitmap = new (ELeave) CFbsBitmap;
	User::LeaveIfError(iTrackNameBitmap->Create(iView->Skin().iTrackNameRect.Size(),
		EColor16M));
	iTrackNameDevice = CFbsBitmapDevice::NewL(iTrackNameBitmap);
	User::LeaveIfError(iTrackNameDevice->CreateContext(iTrackNameContext));

	iTrackTimeBitmap = new (ELeave) CFbsBitmap;
	User::LeaveIfError(iTrackTimeBitmap->Create(iView->Skin().iTrackTimeRect.Size(),
		EColor16M));
	iTrackTimeDevice = CFbsBitmapDevice::NewL(iTrackTimeBitmap);
	User::LeaveIfError(iTrackTimeDevice->CreateContext(iTrackTimeContext));

    CreateWindowL();

#ifdef BEMUSED_SERIES_60

#ifdef POWERPOINT_ONLY
	iFocusIndex = 1; // focus on "Next slide"
#else
	iFocusIndex = 2; // focus on Pause
#endif // POWERPOINT_ONLY

#else

	iFocusIndex = -1; // no button selected

#endif // BEMUSED_SERIES_60

	iHeight = aRect.Height();
	iWidth = aRect.Width();

    SetRect(aRect);
	ActivateL();
    }

CBemusedControlContainer::~CBemusedControlContainer()
    {
	delete iTrackNameContext;
	delete iTrackNameDevice;
	delete iTrackNameBitmap;

	delete iTrackTimeContext;
	delete iTrackTimeDevice;
	delete iTrackTimeBitmap;
    }

void CBemusedControlContainer::SizeChanged()
    {
    }

TInt CBemusedControlContainer::CountComponentControls() const
    {
    return 0;
    }

CCoeControl* CBemusedControlContainer::ComponentControl(TInt /*aIndex*/) const
    {
	return NULL;
    }

// This function should be used to draw the control view following a change in
// internal state. It's mostly used via CBemusedAppUi::DrawControlView().
void CBemusedControlContainer::AppInitiatedDraw() const
	{
	Window().Invalidate();
	ActivateGc();
	Window().BeginRedraw();

	DrawRequiredControls(iAppUi->DrawFlags());

	Window().EndRedraw();
	DeactivateGc();
	}

// This function actually draws the required bits of the screen.
void CBemusedControlContainer::DrawRequiredControls(TUint aRequired) const
	{
	CWindowGc& gc = SystemGc();

	// If the skin isn't valid, just draw a grey rectangle.
	if (!iView->Skin().iValid)
		{
		gc.SetPenStyle(CGraphicsContext::ENullPen);
		gc.SetBrushColor(KRgbGray);
		gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
		gc.DrawRect(Rect());

		iAppUi->SetDrawPending(EFalse);
		iAppUi->SetDrawFlags(KBemusedDrawEverything);

		return;
		}

	// Check which bits of the screen need to be drawn and draw them.

	if (aRequired & KBemusedDrawBackground)
		gc.BitBlt(Rect().iTl, iView->Skin().iBitmap, Rect());

#ifndef POWERPOINT_ONLY
	if (aRequired & KBemusedDrawVolume)
		DrawVolume(Rect());
	if (aRequired & KBemusedDrawTrackTime)
		DrawTrackTime(Rect());
	if (aRequired & KBemusedDrawShuffle)
		DrawShuffle(Rect());
	if (aRequired & KBemusedDrawRepeat)
		DrawRepeat(Rect());
#endif

	if (aRequired & KBemusedDrawTrackName)
		DrawTrackName(Rect());
	if (aRequired & KBemusedDrawButtons)
		DrawButtons(Rect());

	iAppUi->SetDrawPending(EFalse);
	iAppUi->SetDrawFlags(KBemusedDrawEverything);
	}

// When we get a draw request which needs us to redraw everything, do so.
void CBemusedControlContainer::Draw(const TRect& /*aRect*/) const
    {
	DrawRequiredControls(KBemusedDrawEverything);
	}

void CBemusedControlContainer::DrawTrackName(const TRect& aRect) const
	{
	CWindowGc& gc = SystemGc();
	const CFont* font;

	if (iView->Skin().iTrackNameRect.Size() != iTrackNameBitmap->SizeInPixels())
		{
		iTrackNameDevice->Resize(iView->Skin().iTrackNameRect.Size());
		iTrackNameContext->Resized();
		}

	TRect trackNameRect(TPoint(0, 0), iView->Skin().iTrackNameRect.Size());

	// Blit the track name background to the off-screen bitmap.
	iTrackNameContext->BitBlt(TPoint(0, 0), iView->Skin().iBitmap,
		iView->Skin().iTrackNameRect);
	
	font = iView->SkinHandler()->BemusedFontToCFont(iView->Skin().iTrackNameFont);
	if (font)
		iTrackNameContext->UseFont(font);
	
	iTrackNameContext->SetPenColor(iView->Skin().iTrackNameColour);
	
	TInt baseline = iView->Skin().iTrackNameRect.Height() / 2 + font->AscentInPixels() / 2;
	TBuf<256> formatBuf;

	if ((iAppUi->SongState() == ESongPlaying) || (iAppUi->SongState() == ESongPaused))
		{
		if ((!iAppUi->Settings().iUseId3Tags) &&
			(iAppUi->Playlist().Count() > iAppUi->PlaylistIndex()) &&
			(!iAppUi->Shuffle()))
			{
			formatBuf.Copy(*(iAppUi->Playlist()[iAppUi->PlaylistIndex()]));

			// Don't display the file extension for MP3s.
			if ((formatBuf.Length() > 4) && (!formatBuf.Right(4).CompareF(_L(".mp3"))))
				formatBuf.Delete(formatBuf.Length() - 4, 4);
			}
		else
			{
			formatBuf.Copy(iAppUi->SongNameGuess());
			}

		// Check that the track name fits into the space available. If not, we scroll it. There's
		// a small delay at the beginning and end of scrolling.

		TInt trackNameChars = font->TextCount(formatBuf, iView->Skin().iTrackNameRect.Width());

		if (trackNameChars == formatBuf.Length())
			{
			iTrackNameContext->DrawText(formatBuf, /*iView->Skin().iTrackNameRect*/
				trackNameRect, baseline, CGraphicsContext::ECenter, 0);
			}
		else
			{
			TInt offset =
				(iAppUi->CurrentTime() % ((formatBuf.Length() - trackNameChars + 10) / 2)) * 2;
			offset -= 4;

			if (offset < 0)
				offset = 0;
			else if (offset > (formatBuf.Length() - trackNameChars))
				offset = formatBuf.Length() - trackNameChars;

			TPtrC drawPtr = formatBuf.Mid(offset, trackNameChars);

			iTrackNameContext->DrawText(drawPtr, /*iView->Skin().iTrackNameRect*/
				trackNameRect, baseline, CGraphicsContext::ECenter, 0);
			}
		}
	else
		{
		iTrackNameContext->DrawText(_L("Bemused"), /*iView->Skin().iTrackNameRect*/
			trackNameRect, baseline, CGraphicsContext::ECenter, 0);
		}

	iTrackNameContext->DiscardFont();

	// Blit the off-screen bitmap to the screen.
	gc.BitBlt(aRect.iTl + iView->Skin().iTrackNameRect.iTl, iTrackNameBitmap);
	}

void CBemusedControlContainer::DrawTrackTime(const TRect& aRect) const
	{
	CWindowGc& gc = SystemGc();
	const CFont* font;

	if (iView->Skin().iTrackTimeRect.Size() != iTrackTimeBitmap->SizeInPixels())
		{
		iTrackTimeDevice->Resize(iView->Skin().iTrackTimeRect.Size());
		iTrackTimeContext->Resized();
		}

	TRect trackTimeRect(TPoint(0, 0), iView->Skin().iTrackTimeRect.Size());

	iTrackTimeContext->BitBlt(TPoint(0, 0),
		iView->Skin().iBitmap, iView->Skin().iTrackTimeRect);
	
	font = iView->SkinHandler()->BemusedFontToCFont(iView->Skin().iTrackTimeFont);
	if (font)
		iTrackTimeContext->UseFont(font);
	
	iTrackTimeContext->SetPenColor(iView->Skin().iTrackTimeColour);
	
	TInt baseline = iView->Skin().iTrackTimeRect.Height() / 2 + font->AscentInPixels() / 2;

	// Only draw the time while a song's still playing or paused.
	if (((iAppUi->SongState() == ESongPlaying) || (iAppUi->SongState() == ESongPaused)) &&
		(iAppUi->SongLength() > 0) && (iAppUi->CurrentTime() >= 0))
		{
		TBuf<256> formatBuf;

		if (iAppUi->Settings().iTimeDisplay == TBemusedSettings::ECountsUp)
			{
			formatBuf.Format(_L("%02d:%02d"), iAppUi->CurrentTime() / 60,
				iAppUi->CurrentTime() % 60);
			}
		else
			{
			TInt timeLeft = iAppUi->SongLength() - iAppUi->CurrentTime();
			if (timeLeft < 0)
				formatBuf.Copy(_L("--:--"));
			else
				formatBuf.Format(_L("-%02d:%02d"), timeLeft / 60, timeLeft % 60);
			}

		iTrackTimeContext->DrawText(formatBuf, trackTimeRect, baseline,
			CGraphicsContext::ECenter, 0);
		}
	else
		{
		iTrackTimeContext->DrawText(_L("--:--"), trackTimeRect, baseline,
			CGraphicsContext::ECenter, 0);
		}

	iTrackTimeContext->DiscardFont();

	gc.BitBlt(aRect.iTl + iView->Skin().iTrackTimeRect.iTl, iTrackTimeBitmap);
	}

void CBemusedControlContainer::DrawButtons(const TRect& aRect) const
	{
	CWindowGc& gc = SystemGc();

	TInt buttonWidth = iView->Skin().iButtonsRect.Width() / KNumButtons;
	TInt buttonHeight = iView->Skin().iButtonsRect.Height();

	// If the background for the buttons hasn't been drawn, we have to do it ourselves.
	if (!(iAppUi->DrawFlags() & KBemusedDrawBackground))
		{
		gc.BitBlt(aRect.iTl + iView->Skin().iButtonsRect.iTl, iView->Skin().iBitmap,
			iView->Skin().iButtonsRect);
		}

	if (iFocusIndex > -1)
		{
		if (iButtonPressed)
			{
			gc.BitBlt(aRect.iTl + iView->Skin().iButtonsRect.iTl + TPoint(iFocusIndex * buttonWidth, 0),
				iView->Skin().iBitmap,
				TRect(iFocusIndex * buttonWidth, iHeight + buttonHeight, (iFocusIndex + 1) * buttonWidth,
					iHeight + (buttonHeight * 2)));
			}
		else
			{
			gc.BitBlt(aRect.iTl + iView->Skin().iButtonsRect.iTl + TPoint(iFocusIndex * buttonWidth, 0),
				iView->Skin().iBitmap,
				TRect(iFocusIndex * buttonWidth, iHeight, (iFocusIndex + 1) * buttonWidth,
					iHeight + buttonHeight));
			}
		}
	}

void CBemusedControlContainer::DrawShuffle(const TRect& aRect) const
	{
	CWindowGc& gc = SystemGc();

	TInt buttonHeight = iView->Skin().iButtonsRect.Height();

	if (iAppUi->Shuffle())
		{
		gc.BitBlt(aRect.iTl + iView->Skin().iShuffleRect.iTl, iView->Skin().iBitmap,
			TRect((iView->Skin().iVolumeSliderSize.iWidth * 2) + iView->Skin().iRepeatRect.Width(),
				iHeight + (buttonHeight * 2),
				(iView->Skin().iVolumeSliderSize.iWidth * 2) + iView->Skin().iRepeatRect.Width() +
					iView->Skin().iShuffleRect.Width(),
				iHeight + (buttonHeight * 2) + iView->Skin().iShuffleRect.Height()));
		}
	else if (!(iAppUi->DrawFlags() & KBemusedDrawBackground))
		{
		gc.BitBlt(aRect.iTl + iView->Skin().iShuffleRect.iTl,
			iView->Skin().iBitmap, iView->Skin().iShuffleRect);
		}
	}

void CBemusedControlContainer::DrawRepeat(const TRect& aRect) const
	{
	CWindowGc& gc = SystemGc();

	TInt buttonHeight = iView->Skin().iButtonsRect.Height();

	if (iAppUi->Repeat())
		{
		gc.BitBlt(aRect.iTl + iView->Skin().iRepeatRect.iTl, iView->Skin().iBitmap,
			TRect(iView->Skin().iVolumeSliderSize.iWidth * 2, iHeight + (buttonHeight * 2),
				(iView->Skin().iVolumeSliderSize.iWidth * 2) + iView->Skin().iRepeatRect.Width(),
				iHeight + (buttonHeight * 2) + iView->Skin().iRepeatRect.Height()));
		}
	else if (!(iAppUi->DrawFlags() & KBemusedDrawBackground))
		{
		gc.BitBlt(aRect.iTl + iView->Skin().iRepeatRect.iTl,
			iView->Skin().iBitmap, iView->Skin().iRepeatRect);
		}
	}

void CBemusedControlContainer::DrawVolume(const TRect& aRect) const
	{
	CWindowGc& gc = SystemGc();

	// If the background for the volume hasn't been drawn, we have to do it ourselves. This is a bit
	// complicated as we have to draw the whole volume area.
	if (!(iAppUi->DrawFlags() & KBemusedDrawBackground))
		{
		TRect maxVolumeRect;
		GetVolumeSliderArea(maxVolumeRect);

		gc.BitBlt(aRect.iTl + maxVolumeRect.iTl, iView->Skin().iBitmap, maxVolumeRect);
		}

	TInt buttonHeight = iView->Skin().iButtonsRect.Height();

	TPoint volumeDistance = iView->Skin().iVolumeRect.iBr - iView->Skin().iVolumeRect.iTl;
	TInt volumeX = iView->Skin().iVolumeRect.iBr.iX - (volumeDistance.iX * iAppUi->CurrentVolume()) / 255;
	TInt volumeY = iView->Skin().iVolumeRect.iBr.iY - (volumeDistance.iY * iAppUi->CurrentVolume()) / 255;

	// Perform a masked blit of the volume slider onto the background. We have to do this manually
	// because CWindowGc doesn't seem to provide the relevant function.
	for (TInt y = 0; y < iView->Skin().iVolumeSliderSize.iHeight; y++)
		{
		for (TInt x = 0; x < iView->Skin().iVolumeSliderSize.iWidth; x++)
			{
			TRgb colour;
			iView->Skin().iBitmap->GetPixel(colour,
				TPoint(x + iView->Skin().iVolumeSliderSize.iWidth, y + iHeight + (buttonHeight * 2)));
			if (colour != KRgbBlack)
				{
				gc.BitBlt(aRect.iTl + TPoint(volumeX + x, volumeY + y), iView->Skin().iBitmap,
					TRect(x, y + iHeight + (buttonHeight * 2),
						x + 1, y + iHeight + 1 + (buttonHeight * 2)));
				}
			}
		}
	}

void CBemusedControlContainer::HandleControlEventL(CCoeControl* /*aControl*/,
												   TCoeEvent aEventType)
    {
    }

TKeyResponse CBemusedControlContainer::OfferKeyEventL(const TKeyEvent& aKeyEvent,
														TEventCode aType)
	{
	if (!IsFocused())
		return EKeyWasNotConsumed;

	// Check for the joystick being pressed.
	switch (aKeyEvent.iScanCode)
		{
	case EStdKeyDevice3:
		if (aType == EEventKeyDown)
			{
			iButtonPressed = ETrue;
			}
		else if (aType == EEventKeyUp)
			{
			// If we were fast-forwarding or rewinding, we don't want
			// this event to make us go forwards or back a track.
			if (iCheckedButtonPressedAlready)
				{
				iCheckedButtonPressedAlready = EFalse;
				}
			else if (iButtonPressed)
				{
				ExecuteCommand(iFocusIndex);
				}
			iButtonPressed = EFalse;
			}

		iAppUi->DrawControlView(KBemusedDrawButtons);

		return EKeyWasConsumed;
		break;

	default:
		break;
		}

	// Check for key events which move us between buttons.

	switch (aKeyEvent.iCode)
		{
	case EKeyRightArrow:
		if (iFocusIndex < (KNumButtons-1))
			{
			iFocusIndex++;
			iAppUi->DrawControlView(KBemusedDrawButtons);
			return EKeyWasConsumed;
			}
		break;

	case EKeyLeftArrow:
		if (iFocusIndex > 0)
			{
			iFocusIndex--;
			iAppUi->DrawControlView(KBemusedDrawButtons);
			return EKeyWasConsumed;
			}
		break;

	default:
		break;
		}

	// Check for hotkeys.

	if ((aKeyEvent.iCode >= '1') && (aKeyEvent.iCode <= (KNumButtons + '0')))
		{
		iFocusIndex = aKeyEvent.iCode - '1';
		iAppUi->DrawControlView(KBemusedDrawButtons);
		ExecuteCommand(iFocusIndex);

		return EKeyWasConsumed;
		}

	return EKeyWasNotConsumed;
	}

void CBemusedControlContainer::ExecuteCommand(TInt aIndex)
	{
	// Execute one of the commands specified by the buttons.

	switch (aIndex)
		{
	case 0:
		if (iAppUi->PlaylistIndex() > 0)
			iAppUi->PlaylistIndex()--;
		iAppUi->BluetoothHandler()->RequestPreviousTrack();
		break;

#ifdef POWERPOINT_ONLY
	case 2:
#else
	case 1:
#endif
		iAppUi->BluetoothHandler()->RequestPlay();
		break;

#ifndef POWERPOINT_ONLY
	case 2:
		iAppUi->BluetoothHandler()->RequestPause();
		break;
#endif

	case 3:
		iAppUi->BluetoothHandler()->RequestStop((TActionOnStopping)
			iAppUi->Settings().iActionOnStopping);
		break;

#ifdef POWERPOINT_ONLY
	case 1:
#else
	case 4:
#endif
		if (iAppUi->PlaylistIndex() < (iAppUi->Playlist().Count()-1))
			iAppUi->PlaylistIndex()++;
		iAppUi->BluetoothHandler()->RequestNextTrack();
		break;

	default:
		break;
		}
	}

TBool CBemusedControlContainer::CheckFastForwardAndRewind()
	{
	// If the user holds down the fast forward or rewind buttons, perform the
	// relevant action.

	if (iButtonPressed)
		{
		if (iCheckedButtonPressedAlready)
			{
			if (iFocusIndex == 0)
				{
				iAppUi->BluetoothHandler()->RequestRewind();
				return ETrue;
				}
			else if (iFocusIndex == 4)
				{
				iAppUi->BluetoothHandler()->RequestFastForward();
				return ETrue;
				}
			}
		else
			{
			if ((iFocusIndex == 0) || (iFocusIndex == 4))
				iCheckedButtonPressedAlready = ETrue;
			}
		}
	else
		{
		iCheckedButtonPressedAlready = EFalse;
		}
	return EFalse;
	}

void CBemusedControlContainer::GetVolumeSliderArea(TRect& aVolumeSliderArea) const
	{
	if (iView->Skin().iVolumeRect.iTl.iX < iView->Skin().iVolumeRect.iBr.iX)
		{
		aVolumeSliderArea.iTl.iX = iView->Skin().iVolumeRect.iTl.iX;
		aVolumeSliderArea.iBr.iX = iView->Skin().iVolumeRect.iBr.iX;
		}
	else
		{
		aVolumeSliderArea.iTl.iX = iView->Skin().iVolumeRect.iBr.iX;
		aVolumeSliderArea.iBr.iX = iView->Skin().iVolumeRect.iTl.iX;
		}

	if (iView->Skin().iVolumeRect.iTl.iY < iView->Skin().iVolumeRect.iBr.iY)
		{
		aVolumeSliderArea.iTl.iY = iView->Skin().iVolumeRect.iTl.iY;
		aVolumeSliderArea.iBr.iY = iView->Skin().iVolumeRect.iBr.iY;
		}
	else
		{
		aVolumeSliderArea.iTl.iY = iView->Skin().iVolumeRect.iBr.iY;
		aVolumeSliderArea.iBr.iY = iView->Skin().iVolumeRect.iTl.iY;
		}
	aVolumeSliderArea.iBr.iX += iView->Skin().iVolumeSliderSize.iWidth;
	aVolumeSliderArea.iBr.iY += iView->Skin().iVolumeSliderSize.iHeight;
	}

////////////////////////////////////////////////////////////////////////////////
// UIQ-only code below this point.
////////////////////////////////////////////////////////////////////////////////

#ifndef BEMUSED_SERIES_60

void CBemusedControlContainer::HandlePointerEventL(const TPointerEvent& aPointerEvent)
	{
	if ((aPointerEvent.iType == TPointerEvent::EButton1Up) ||
		(aPointerEvent.iType == TPointerEvent::EDrag))
		{
		iButtonPressed = EFalse;
		}

	TRect volumeSliderArea;
	GetVolumeSliderArea(volumeSliderArea);

	if (volumeSliderArea.Contains(aPointerEvent.iPosition))
		{
		TPoint pointFromStart =
			AbsolutePoint(aPointerEvent.iPosition - iView->Skin().iVolumeRect.iTl);

		TInt distFromStart = pointFromStart.iX + pointFromStart.iY;
		
		TPoint pointFromEnd =
			AbsolutePoint(aPointerEvent.iPosition - iView->Skin().iVolumeRect.iBr);

		TInt distFromEnd = pointFromEnd.iX + pointFromEnd.iY;

		TInt totalDistance = distFromStart + distFromEnd;

		TInt newVolume = (distFromEnd * 256) / totalDistance;
		if (newVolume >= 256)
			newVolume = 255;

		iAppUi->SetVolume(newVolume);
		iAppUi->DrawControlView(KBemusedDrawVolume);
		}
	else if (iView->Skin().iButtonsRect.Contains(aPointerEvent.iPosition))
		{
		TInt xDistance = aPointerEvent.iPosition.iX - iView->Skin().iButtonsRect.iTl.iX;
		TReal ratio = ((TReal) xDistance) / ((TReal) iView->Skin().iButtonsRect.Width());

		if (ratio >= 1.0)
			ratio = 0.9;

		if (aPointerEvent.iType == TPointerEvent::EButton1Down)
			{
			iFocusIndex = (TInt)(ratio * KNumButtons);
			iButtonPressed = ETrue;

			iAppUi->DrawControlView(KBemusedDrawButtons);
			}
		else if (aPointerEvent.iType == TPointerEvent::EButton1Up)
			{
			iFocusIndex = (TInt)(ratio * KNumButtons);
			iButtonPressed = EFalse;

			// Avoid a spurious event after fast-forwarding or rewinding.
			if (!iCheckedButtonPressedAlready)
				ExecuteCommand(iFocusIndex);

			iFocusIndex = -1;

			iAppUi->DrawControlView(KBemusedDrawButtons);
			}
		}
	else if (iView->Skin().iVolumeRect.Contains(aPointerEvent.iPosition))
		{
		TInt yDistance = aPointerEvent.iPosition.iY - iView->Skin().iVolumeRect.iTl.iY;
		TReal ratio = ((TReal) yDistance) / ((TReal) iView->Skin().iVolumeRect.Height());

		TInt newVolume = (TInt)(ratio * 256);
		if (newVolume >= 256)
			newVolume = 255;

		if (aPointerEvent.iType == TPointerEvent::EButton1Down)
			{
			iAppUi->SetVolume(newVolume);
			iAppUi->DrawControlView(KBemusedDrawVolume);
			}
		}
	else if (iView->Skin().iRepeatRect.Contains(aPointerEvent.iPosition))
		{
		iAppUi->HandleCommandL(EBemusedCmdRepeat);
		}
	else if (iView->Skin().iShuffleRect.Contains(aPointerEvent.iPosition))
		{
		iAppUi->HandleCommandL(EBemusedCmdShuffle);
		}
	else if (iView->Skin().iTrackTimeRect.Contains(aPointerEvent.iPosition))
		{
		if (aPointerEvent.iType == TPointerEvent::EButton1Down)
			{
			if (iAppUi->Settings().iTimeDisplay == TBemusedSettings::ECountsUp)
				iAppUi->Settings().iTimeDisplay = TBemusedSettings::ECountsDown;
			else
				iAppUi->Settings().iTimeDisplay = TBemusedSettings::ECountsUp;

			iAppUi->DrawControlView(KBemusedDrawTrackTime);
			}
		}
	}

TPoint CBemusedControlContainer::AbsolutePoint(const TPoint& aPoint)
	{
	TPoint point;

	if (aPoint.iX < 0)
		point.iX = -aPoint.iX;
	else
		point.iX = aPoint.iX;

	if (aPoint.iY < 0)
		point.iY = -aPoint.iY;
	else
		point.iY = aPoint.iY;

	return point;
	}

#endif // !BEMUSED_SERIES_60

// End of File  
