/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#include <eikenv.h>

#include <Bemused.rsg>
#include "BemusedSkinHandler.h"
#include "BemusedAppUi.h"

#ifdef BEMUSED_SERIES_60
  #include "BemusedControlView.h"
  #include "BemusedBrowseView.h"
  #include "BemusedPlaylistView.h"
#else
  #include <e32math.h>
  #include "BemusedMainView.h"
  #include "BemusedInstallSkinDialog.h"
#endif

#include "BemusedControlContainer.h"

CBemusedSkinHandler::CBemusedSkinHandler(CBemusedAppUi* aAppUi, TBemusedSkin& aSkin)
	: iAppUi(aAppUi), iSkin(aSkin)
	{
	}

CBemusedSkinHandler* CBemusedSkinHandler::NewL(CBemusedAppUi* aAppUi, TBemusedSkin& aSkin)
	{
	CBemusedSkinHandler* self = new (ELeave) CBemusedSkinHandler(aAppUi, aSkin);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();

	return self;
	}

void CBemusedSkinHandler::ConstructL()
    {
	// Create a bitmap suitable for holding a 24-bit skin image.
	iSkin.iBitmap = new (ELeave) CFbsBitmap;
	User::LeaveIfError(
		iSkin.iBitmap->Create(TSize(KMaxSkinBitmapWidth, KMaxSkinBitmapHeight), EColor16M));
	
	iSkinLoader = CMdaImageFileToBitmapUtility::NewL(*this);

#ifndef BEMUSED_SERIES_60
	iSkinScaler = CMdaBitmapScaler::NewL();

	CEikonEnv::Static()->FsSession().MkDirAll(KCDriveSkinTextPath);
	CEikonEnv::Static()->FsSession().MkDirAll(KCDriveSkinImagePath);
	CEikonEnv::Static()->FsSession().MkDirAll(KDDriveSkinTextPath);
	CEikonEnv::Static()->FsSession().MkDirAll(KDDriveSkinImagePath);
#endif

	// Try to load the skin. The operation is asynchronous; this is only the first stage.
	TRAPD(err, LoadSkinL());
	if (err != KErrNone)
		{
		TBuf<256> resourceBuf;
		iAppUi->CoeEnv()->ReadResource(resourceBuf, R_LOAD_SKIN_ERROR);
		CEikonEnv::Static()->InfoWinL(resourceBuf, KNullDesC);
		}
	}

CBemusedSkinHandler::~CBemusedSkinHandler()
    {
	iSkinLoader->Close();
	delete iSkinLoader;
	delete iSkinScaler;
	delete iSkin.iBitmap;
    }

void CBemusedSkinHandler::MiuoConvertComplete(TInt aError)
	{
	if (!iScaling)
		{
		iSkinLoader->Close();

#ifdef BEMUSED_SERIES_60

		if (aError != KErrNone)
			{
			TBuf<256> resourceBuf;
			iAppUi->CoeEnv()->ReadResource(resourceBuf, R_LOAD_SKIN_ERROR);
			TRAPD(err, CEikonEnv::Static()->InfoWinL(KNullDesC, resourceBuf));
			}
		else
			{
			iSkin.iValid = ETrue;
			}

		if (iAppUi->ControlView()->ControlContainer())
			iAppUi->ControlView()->ControlContainer()->DrawNow();
#else
		TSize currentSize = iSkin.iBitmap->SizeInPixels();
		TReal width = currentSize.iWidth;
		TReal height = currentSize.iHeight;
		width *= KSkinScalingFactor;
		height *= KSkinScalingFactor;

		TSize newSize(width, height);

		iSkinScaler->ScaleL(*this, *iSkin.iBitmap, newSize, EFalse);

		iScaling = ETrue;
#endif
		}
	else
		{
		if (aError != KErrNone)
			{
			TBuf<256> resourceBuf;
			iAppUi->CoeEnv()->ReadResource(resourceBuf, R_LOAD_SKIN_ERROR);
			TRAPD(err, CEikonEnv::Static()->InfoWinL(KNullDesC, resourceBuf));
			}
		else
			{
			iSkin.iValid = ETrue;
			}

		iAppUi->DrawControlView(KBemusedDrawEverything);
		iScaling = EFalse;
		}
	}

void CBemusedSkinHandler::MiuoCreateComplete(TInt /*aError*/)
	{
	}

void CBemusedSkinHandler::MiuoOpenComplete(TInt aError)
	{
	if (aError == KErrNone)
		{
		TRAP(aError, iSkinLoader->ConvertL(*iSkin.iBitmap));
		if (aError != KErrNone)
			{
			TBuf<256> resourceBuf;
			iAppUi->CoeEnv()->ReadResource(resourceBuf, R_LOAD_SKIN_ERROR);
			TRAP(aError, CEikonEnv::Static()->InfoWinL(resourceBuf, KNullDesC));
			}
		}
	}

void CBemusedSkinHandler::LoadSkinL()
	{
	iSkin.iValid = EFalse;

	CDir* entryList;
	TBuf<256> formatBuf;
	TFileName skinPath;
	TInt ret;

	TBemusedSettings::TDisplayType displayType = (TBemusedSettings::TDisplayType)
		iAppUi->Settings().iDisplayType;

	TInt skinIndex = iAppUi->Settings().iSkinFileIndex;

	if (displayType == TBemusedSettings::EInWindow)
		skinPath.Format(_L("%S\\*_iw.png"), &iAppUi->SkinPathL());
	else
		skinPath.Format(_L("%S\\*_fs.png"), &iAppUi->SkinPathL());

	ret = CEikonEnv::Static()->FsSession().GetDir(skinPath, KEntryAttNormal,
		ESortByName, entryList);

	if (ret != KErrNone)
		{
		delete entryList;

		TBuf<256> resourceBuf;
		iAppUi->CoeEnv()->ReadResource(resourceBuf, R_SKIN_LIST_ERROR);
		formatBuf.Format(resourceBuf, ret);
		CEikonEnv::Static()->InfoWinL(formatBuf, KNullDesC);
		return;
		}

	CleanupStack::PushL(entryList);

	if (entryList->Count() <= skinIndex)
		{
		iAppUi->CoeEnv()->ReadResource(formatBuf, R_LOAD_SKIN_ERROR);
		CEikonEnv::Static()->InfoWinL(formatBuf, KNullDesC);
		}
	else
		{
		const TDesC& entryName = (*entryList)[skinIndex].iName;
		TPtrC entryWithoutExt = entryName.Left(entryName.Length() - 4);

		iAppUi->SetSkinName(entryWithoutExt.Left(entryWithoutExt.Length() - 3));

		formatBuf.Format(_L("%S\\%S"), &iAppUi->SkinPathL(), &entryName);
		iSkinLoader->OpenL(formatBuf);

		formatBuf.Format(_L("%S\\%S.txt"), &iAppUi->SkinPathL(), &entryWithoutExt);
		ReadSkinDescriptionL(formatBuf);

		iAppUi->BrowseView()->ReadSkinDescriptionL(formatBuf);
		iAppUi->PlaylistView()->ReadSkinDescriptionL(formatBuf);
		}

	CleanupStack::PopAndDestroy(); // entryList
	}

void CBemusedSkinHandler::ReadSkinDescriptionL(const TDesC& aFileName)
	{
	// Set default values in case we can't open the skin.

	iSkin.iButtonsRect.SetRect(0, 105, 175, 140);
	iSkin.iTrackNameRect.SetRect(10, 10, 165, 36);
	iSkin.iTrackTimeRect.SetRect(10, 40, 165, 76);
	iSkin.iShuffleRect.SetRect(10, 80, 20, 90);
	iSkin.iRepeatRect.SetRect(10, 100, 20, 110);
	iSkin.iVolumeRect.SetRect(170, 80, 175, 120);
	iSkin.iTrackNameColour = KRgbBlack;
	iSkin.iTrackTimeColour = KRgbBlack;
	iSkin.iTrackNameFont = EBemusedTitleFont;
	iSkin.iTrackTimeFont = EBemusedTitleFont;
	iAppUi->SetSkinAuthor(KNullDesC8);

	RFile file;
	User::LeaveIfError(file.Open(CEikonEnv::Static()->FsSession(),
		aFileName, EFileShareAny | EFileRead));
	CleanupClosePushL(file);

	TInt fileSize;
	User::LeaveIfError(file.Size(fileSize));

	HBufC8* fileBuf = HBufC8::NewLC(fileSize);
	TPtr8 fileBufPtr = fileBuf->Des();

	User::LeaveIfError(file.Read(fileBufPtr));

	TLex8 lex(*fileBuf);
	TPtrC8 ptr(lex.NextToken());

	while (ptr != KNullDesC8)
		{
		lex.SkipSpace();

		if (!ptr.CompareF(_L8("buttons:")))
			{
			ReadSkinRect(lex, iSkin.iButtonsRect);
			}
		else if (!ptr.CompareF(_L8("trackname:")))
			{
			ReadSkinRect(lex, iSkin.iTrackNameRect);
			}
		else if (!ptr.CompareF(_L8("tracktime:")))
			{
			ReadSkinRect(lex, iSkin.iTrackTimeRect);
			}
		else if (!ptr.CompareF(_L8("shuffle:")))
			{
			ReadSkinRect(lex, iSkin.iShuffleRect);
			}
		else if (!ptr.CompareF(_L8("repeat:")))
			{
			ReadSkinRect(lex, iSkin.iRepeatRect);
			}
		else if (!ptr.CompareF(_L8("volume:")))
			{
			ReadSkinRect(lex, iSkin.iVolumeRect);

			lex.Val(iSkin.iVolumeSliderSize.iWidth);
			lex.SkipSpace();
			lex.Val(iSkin.iVolumeSliderSize.iHeight);
			lex.SkipSpace();

#ifndef BEMUSED_SERIES_60
			TReal roundedValue;

			TReal value = iSkin.iVolumeSliderSize.iWidth;
			value *= KSkinScalingFactor;
			Math::Round(roundedValue, value, 0);

			iSkin.iVolumeSliderSize.iWidth = (TInt) roundedValue;

			value = iSkin.iVolumeSliderSize.iHeight;
			value *= KSkinScalingFactor;
			Math::Round(roundedValue, value, 0);

			iSkin.iVolumeSliderSize.iHeight = (TInt) roundedValue;
#endif
			}
		else if (!ptr.CompareF(_L8("tracknamecolour:")))
			{
			ReadSkinColour(lex, iSkin.iTrackNameColour);
			}
		else if (!ptr.CompareF(_L8("tracktimecolour:")))
			{
			ReadSkinColour(lex, iSkin.iTrackTimeColour);
			}
		else if (!ptr.CompareF(_L8("tracknamefont:")))
			{
			TInt font;
			lex.Val(font);
			iSkin.iTrackNameFont = (TBemusedFont) font;
			lex.SkipSpace();
			}
		else if (!ptr.CompareF(_L8("tracktimefont:")))
			{
			TInt font;
			lex.Val(font);
			iSkin.iTrackTimeFont = (TBemusedFont) font;
			lex.SkipSpace();
			}
		else if (!ptr.CompareF(_L8("author:")))
			{
			TChar c;
			TBuf8<128> authorBuf;

			c = lex.Get();
			while ((c != '\r') && (c != '\n') && (c != 0))
				{
				authorBuf.Append(c);
				c = lex.Get();
				}
				
			iAppUi->SetSkinAuthor(authorBuf);
			lex.SkipSpace();
			}
		else
			{
			lex.SkipSpace();
			}
		ptr.Set(lex.NextToken());
		}

	CleanupStack::PopAndDestroy(2); // file, fileBuf
	}

void CBemusedSkinHandler::ReadSkinColour(TLex8& aLex, TRgb& aColour)
	{
	TInt colour;

	aLex.Val(colour);
	aColour.SetRed(colour);
	aLex.SkipSpace();

	aLex.Val(colour);
	aColour.SetGreen(colour);
	aLex.SkipSpace();

	aLex.Val(colour);
	aColour.SetBlue(colour);
	aLex.SkipSpace();
	}

void CBemusedSkinHandler::ReadSkinRect(TLex8& aLex, TRect& aRect)
	{
	aLex.Val(aRect.iTl.iX);
	aLex.SkipSpace();
	aLex.Val(aRect.iTl.iY);
	aLex.SkipSpace();
	aLex.Val(aRect.iBr.iX);
	aLex.SkipSpace();
	aLex.Val(aRect.iBr.iY);
	aLex.SkipSpace();

#ifndef BEMUSED_SERIES_60
	TReal roundedValue;

	TReal value = aRect.iTl.iX;
	value *= KSkinScalingFactor;
	Math::Round(roundedValue, value, 0);
	aRect.iTl.iX = (TInt) roundedValue;

	value = aRect.iTl.iY;
	value *= KSkinScalingFactor;
	Math::Round(roundedValue, value, 0);
	aRect.iTl.iY = (TInt) roundedValue;

	value = aRect.iBr.iX;
	value *= KSkinScalingFactor;
	Math::Round(roundedValue, value, 0);
	aRect.iBr.iX = (TInt) roundedValue;

	value = aRect.iBr.iY;
	value *= KSkinScalingFactor;
	Math::Round(roundedValue, value, 0);
	aRect.iBr.iY = (TInt) roundedValue;
#endif
	}

const CFont* CBemusedSkinHandler::BemusedFontToCFont(TBemusedFont aBemusedFont)
	{
	CEikonEnv* eikonEnv = CEikonEnv::Static();

	switch (aBemusedFont)
		{
	case EBemusedTitleFont:
		return eikonEnv->TitleFont();
	case EBemusedDenseFont:
		return eikonEnv->DenseFont();
	case EBemusedLegendFont:
		return eikonEnv->LegendFont();
	case EBemusedAnnotationFont:
		return eikonEnv->AnnotationFont();
	case EBemusedSymbolFont:
#ifdef BEMUSED_SERIES_60
		return eikonEnv->SymbolFont();
#else
		return eikonEnv->NormalFont();
#endif
	case EBemusedNormalFont:
		return eikonEnv->NormalFont();
	default:
		break;
		}
	return NULL;
	}

// End of File

