/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#include <eikenv.h>
#include <eikappui.h>

#include "BemusedApp.h"
#include "BluetoothHandler.h"
#include "BemusedAppUi.h"

#ifdef BEMUSED_SERIES_60
  #include <Bemused.rsg>
#else
  #include <Bemused.rsg>
  #include <QBTselectdlg.h>
  #include <btdevice.h>
  #include "BemusedProgressDialog.h"
#endif

#include <in_sock.h>

_LIT(KSongList, "songlist.dat");

CBluetoothHandler::CBluetoothHandler(MBluetoothListener* aListener,
									 const TBTDevAddr& aAutoConnectAddress,
									 TInt aAutoConnectPort)
: CActive(EPriorityStandard), iListener(aListener),
  iAutoConnectAddress(aAutoConnectAddress), iAutoConnectPort(aAutoConnectPort)
	{
	CActiveScheduler::Add(this);

	iServiceClass = KSerialPortService;
	}

CBluetoothHandler* CBluetoothHandler::NewL(MBluetoothListener* aListener,
										   const TBTDevAddr& aAutoConnectAddress,
										   TInt aAutoConnectPort)
	{
	CBluetoothHandler* self = new (ELeave) CBluetoothHandler(aListener,
		aAutoConnectAddress, aAutoConnectPort);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(); // self

	return self;
	}

void CBluetoothHandler::ConstructL()
	{
	iCommandTimer = CPeriodic::NewL(EPriorityStandard);

	User::LeaveIfError(iSocketServ.Connect());
	User::LeaveIfError(iResolver.Open(iSocketServ, KAfInet, KProtocolInetUdp));
	User::LeaveIfError(iNotifier.Connect());

	// Get the path to the songlist
	TFileName app = CEikonEnv::Static()->EikAppUi()->Application()->AppFullName();
	TParsePtr parse(app);
	TFileName songListFileName(parse.DriveAndPath());
	songListFileName.Append(KSongList);

	ReadSongListFile(songListFileName);
	}

CBluetoothHandler::~CBluetoothHandler()
	{
	Cancel();

	CloseAllResources();

	delete iCommandTimer;
#ifdef BEMUSED_SERIES_60
	delete iWaitDialog;
#endif
	delete iProgressDialog;
	delete iDirTreeRoot;
	delete iSdpSearchPattern;
	delete iSdpAgent;

	iResolver.Close();
	iSocketServ.Close();
	iNotifier.Close();
	}

void CBluetoothHandler::CloseAllResources()
	{
	iState = EIdle;
	iConnected = EFalse;
	iCurrentCommand = EBemusedNone;
	iQueuedCommand = EBemusedNone;
	iWaitingForResponseData = EFalse;

	if (iCommandTimer)
		iCommandTimer->Cancel();

#ifdef BEMUSED_SERIES_60
	if (iWaitDialog)
		iWaitDialog->ProcessFinishedL();

	if (iProgressDialog)
		iProgressDialog->ProcessFinishedL();
#else
	CEikonEnv::Static()->BusyMsgCancel();

	delete iProgressDialog;
	iProgressDialog = NULL;
#endif

	iSongListFile.Close();
	iDownloadFile.Close();
	iSocket.Close();
	}

void CBluetoothHandler::ReadSongListFile(const TDesC& aFileName)
	{
	TInt ret;

	ret = iSongListFile.Open(CEikonEnv::Static()->FsSession(), aFileName, EFileRead | EFileShareAny);
	if (ret != KErrNone)
		{
		if (iListener->Settings().iRetrieveList == TBemusedSettings::EAllAtOnce)
			RequestList(ETrue);
		else
			RequestList(EFalse);

		return;
		}

	delete iDirTreeRoot;
	iDirTreeRoot = NULL;
	iDirTreeCurrentNode = NULL;

	iTotalPacketBuf.Zero();

	iState = EReadingSongListFile;
	iSongListFile.Read(iPacketBuf, iStatus);
	SetActive();
	}

void CBluetoothHandler::Connect()
	{
	iCurrentCommand = EBemusedConnect;
	iState = ESelectingServer;

	if (!iSocket.SubSessionHandle())
		{
		TInt ret;

		if (iListener->Settings().iConnectUsing == TBemusedSettings::EBluetooth)
			ret = iSocket.Open(iSocketServ, KBTAddrFamily, KSockStream, KRFCOMM);
		else
			ret = iSocket.Open(iSocketServ, KAfInet, KSockStream, KProtocolInetTcp);

		if (ret != KErrNone)
			{
			ConnectionFailed(ret);
			return;
			}
		}

/*	TBuf<256> formatBuf;
	formatBuf.Format(_L("Last addr %d %d %d %d %d %d"),
		iAutoConnectAddress[0],iAutoConnectAddress[1],iAutoConnectAddress[2],
		iAutoConnectAddress[3],iAutoConnectAddress[4],iAutoConnectAddress[5]);
	CEikonEnv::Static()->InfoWinL(formatBuf, KNullDesC);
*/
	TBTDevAddr zeroAddr;

	if (iListener->Settings().iConnectUsing == TBemusedSettings::EBluetooth)
		{
		if ((iQueuedCommand == EBemusedFindServer) ||
			(iListener->Settings().iConnectToPc != TBemusedSettings::EAutoToSamePc) ||
			(iAutoConnectAddress == zeroAddr))
			{
			// If we get here, we need to do a device discovery.

#ifdef BEMUSED_SERIES_60
			iNotifier.StartNotifierAndGetResponse(iStatus, KDeviceSelectionNotifierUid,
				iDeviceSelectionPckg, iDeviceFoundPckg);
#else
			TBTDevAddr devAddr;
			TBTDeviceName devName;
			TBTDeviceClass devClass;
			TInt serviceFilter = -1;
			CQBTUISelectDialog* dialog = new (ELeave) CQBTUISelectDialog(devAddr, devName, devClass,
						CQBTUISelectDialog::EQBTDeviceFilterAll, serviceFilter/*, ETrue*/);

			// Launch "Select Bluetooth device" dialog.
			if (dialog->LaunchSingleSelectDialogLD())
				{
				iDeviceFoundPckg().SetDeviceAddress(devAddr);
				iDeviceFoundPckg().SetDeviceName(devName);
				iDeviceFoundPckg().SetDeviceClass(devClass);

				// Get rid of any previously held errors.
				iStatus = KErrNone;
				RunL();
				}
			else
				{
				CloseAllResources();
				}
			return;
#endif // BEMUSED_SERIES_60
			}
		else
			{
			// If we get here, we want to do an automatic connect to the same
			// PC, using Bluetooth.
			iState = EConnecting;

			TBTSockAddr addr;
			addr.SetBTAddr(iAutoConnectAddress);
			addr.SetPort(iAutoConnectPort);
			iDeviceFoundPckg().SetDeviceAddress(iAutoConnectAddress);
			iRemotePort = iAutoConnectPort;

			iSocket.Connect(addr, iStatus);
			}
		}
	else if (iListener->Settings().iConnectUsing == TBemusedSettings::EMrouter)
		{
		// If we get here, we need to do a DNS lookup of mRouter.
		iResolver.GetByName(_L("wsockhost.mrouter"), iNameEntry, iStatus);
		}
	else
		{
		iResolver.GetByName(iListener->Settings().iServerAddress, iNameEntry, iStatus);
		}

	SetActive();
	}

void CBluetoothHandler::WriteCommand()
	{
	iWaitingForResponseData = EFalse;

	if (iConnected)
		{
		if (iVersionState == ENotGotVersion)
			{
			iQueuedCommand = iCurrentCommand;
			iCurrentCommand = EBemusedGetVersion;

			iVersionState = EGettingVersion;
			iVersionQueuedPacketBuf.Copy(iPacketBuf);
			iPacketBuf.Copy(_L8("VERS"));

			if (iCommandTimer->IsActive())
				iCommandTimer->Cancel();

			iCommandTimer->Start(2 * 1000000, 1 * 1000000,
				TCallBack(CommandTimerCallBack, this));

			iState = EWriting;
			iSocket.Write(iPacketBuf, iStatus);
			SetActive();
			}
		else
			{
			if (!iCommandTimer->IsActive() && (iCurrentCommand != EBemusedDownloadFile))
				{
				if ((iCurrentCommand != EBemusedList) &&
					(iCurrentCommand != EBemusedGetPlaylist))
					{
					iCommandTimer->Start(KNormalCommandTimeout, 1 * 1000000,
						TCallBack(CommandTimerCallBack, this));
					}
				else
					{
					iCommandTimer->Start(KLongCommandTimeout, 1 * 1000000,
						TCallBack(CommandTimerCallBack, this));
					}
				}
			iState = EWriting;
			iSocket.Write(iPacketBuf, iStatus);
			SetActive();
			}
		}
	else
		{
		iQueuedCommand = iCurrentCommand;
		Connect();
		}
	}

void CBluetoothHandler::RequestList(TBool aListEntireTree)
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	if (iConnected)
		{
		TRAPD(err, ShowWaitDialogL(R_REFRESHING_LIST_MSG));
		}

	delete iDirTreeRoot;
	iDirTreeRoot = NULL;
	iDirTreeCurrentNode = NULL;
	iDirTreeNodeToExpand = NULL;

	iCurrentCommand = EBemusedList;

	if (!aListEntireTree)
		{
		iPacketBuf.Copy(_L8("DLST  "));
		iPacketBuf[4] = 0;
		iPacketBuf[5] = 0;
		}
	else
		{
		iPacketBuf.Copy(_L8("LIST"));
		}
	WriteCommand();
	}

void CBluetoothHandler::RequestGetPlaylist()
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	if (iConnected)
		{
		TRAPD(err, ShowWaitDialogL(R_GETTING_PLAYLIST_MSG));
		}

	iCurrentCommand = EBemusedGetPlaylist;
	iPacketBuf.Copy(_L8("PLST"));
	WriteCommand();
	}

void CBluetoothHandler::RequestDirectoryList(CDirTreeNode* aNode)
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	if (iConnected)
		{
		TRAPD(err, ShowWaitDialogL(R_LISTING_FOLDER_MSG));
		}

	iDirTreeCurrentNode = iDirTreeNodeToExpand = aNode;

	iCurrentCommand = EBemusedList;

	TBuf8<256> fileName = *aNode->iFileName;

	do {
		aNode = aNode->iParent;

		// Allow for the case where the root has no filename.
		if (aNode && (*aNode->iFileName != KNullDesC8))
			{
			fileName.Insert(0, _L8("\\"));
			fileName.Insert(0, *aNode->iFileName);
			}
		} while (aNode && aNode->iParent);

	iPacketBuf.Format(_L8("DLST  %S"), &fileName);
	iPacketBuf[4] = (TUint8)((fileName.Length() >> 8) & 0xFF);
	iPacketBuf[5] = (TUint8)(fileName.Length() & 0xFF);

	WriteCommand();
	}

void CBluetoothHandler::RequestInfo()
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	iCurrentCommand = EBemusedCurrentTrackInfo;

	iPacketBuf.Copy(_L8("INFO"));
	WriteCommand();
	}

void CBluetoothHandler::RequestPlayFile(const TDesC8& aFileName)
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	iCurrentCommand = EBemusedPlayFile;

	iPacketBuf.Copy(_L8("PLAY"));
	iPacketBuf.Append(_L8("  "));
	iPacketBuf[4] = (TUint8)((aFileName.Length() >> 8) & 0xFF);
	iPacketBuf[5] = (TUint8)(aFileName.Length() & 0xFF);
	iPacketBuf.Append(aFileName);

	WriteCommand();
	}

void CBluetoothHandler::RequestAddFileToPlaylist(const TDesC8& aFileName)
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	iCurrentCommand = EBemusedAddFileToPlaylist;

	iPacketBuf.Copy(_L8("LADD"));
	iPacketBuf.Append(_L8("  "));
	iPacketBuf[4] = (TUint8) ((aFileName.Length() >> 8) & 0xFF);
	iPacketBuf[5] = (TUint8) (aFileName.Length() & 0xFF);
	iPacketBuf.Append(aFileName);

	WriteCommand();
	}

void CBluetoothHandler::RequestFileInfo(const TDesC8& aFileName)
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	iCurrentCommand = EBemusedFileInfo;
	iDownloadFileName.Copy(aFileName);

	iPacketBuf.Copy(_L8("FINF"));
	iPacketBuf.Append(_L8("  "));
	iPacketBuf[4] = (TUint8) ((aFileName.Length() >> 8) & 0xFF);
	iPacketBuf[5] = (TUint8) (aFileName.Length() & 0xFF);
	iPacketBuf.Append(aFileName);

	WriteCommand();
	}

void CBluetoothHandler::RequestDownloadFile(const TDesC8& aFileName)
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	iCurrentCommand = EBemusedDownloadFile;
	iDownloadFileName.Copy(aFileName);

	iPacketBuf.Copy(_L8("DOWN"));
	iPacketBuf.Append(_L8("  "));
	iPacketBuf[4] = (TUint8) ((aFileName.Length() >> 8) & 0xFF);
	iPacketBuf[5] = (TUint8) (aFileName.Length() & 0xFF);
	iPacketBuf.Append(aFileName);

	WriteCommand();
	}

void CBluetoothHandler::RequestSetVolume(TUint8 aVolume)
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	iCurrentCommand = EBemusedSetVolume;

	iPacketBuf.Copy(_L8("VOLM"));
	iPacketBuf.Append(_L8(" "));
	iPacketBuf[4] = aVolume;

	WriteCommand();
	}

void CBluetoothHandler::RequestStop(TActionOnStopping aAction)
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	iCurrentCommand = EBemusedStop;

	if (aAction == EFadeOut)
		iPacketBuf.Copy(_L8("FADE"));
	else if (aAction == EStopAtEndOfSong)
		iPacketBuf.Copy(_L8("STEN"));
	else
		iPacketBuf.Copy(_L8("STOP"));

	WriteCommand();
	}

void CBluetoothHandler::RequestPause()
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	iCurrentCommand = EBemusedPause;
	iPacketBuf.Copy(_L8("PAUS"));
	WriteCommand();
	}

void CBluetoothHandler::RequestPlay()
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	iCurrentCommand = EBemusedPlay;
	iPacketBuf.Copy(_L8("STRT"));
	WriteCommand();
	}

void CBluetoothHandler::RequestNextTrack()
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	iCurrentCommand = EBemusedNextTrack;
	iPacketBuf.Copy(_L8("NEXT"));
	WriteCommand();
	}

void CBluetoothHandler::RequestPreviousTrack()
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	iCurrentCommand = EBemusedPreviousTrack;
	iPacketBuf.Copy(_L8("PREV"));
	WriteCommand();
	}

void CBluetoothHandler::RequestFastForward()
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	iCurrentCommand = EBemusedFastForward;
	iPacketBuf.Copy(_L8("FFWD"));
	WriteCommand();
	}

void CBluetoothHandler::RequestRewind()
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	iCurrentCommand = EBemusedRewind;
	iPacketBuf.Copy(_L8("RWND"));
	WriteCommand();
	}

void CBluetoothHandler::RequestShuffle(TBool aShuffleOn)
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	iCurrentCommand = EBemusedShuffle;
	iPacketBuf.Copy(_L8("SHFL "));
	iPacketBuf[4] = (TUint8) aShuffleOn;
	WriteCommand();
	}

void CBluetoothHandler::RequestRepeat(TBool aRepeatOn)
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	iCurrentCommand = EBemusedRepeat;
	iPacketBuf.Copy(_L8("REPT "));
	iPacketBuf[4] = (TUint8) aRepeatOn;
	WriteCommand();
	}

void CBluetoothHandler::RequestShutDown()
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	iCurrentCommand = EBemusedShutDown;
	iPacketBuf.Copy(_L8("SHUT"));
	WriteCommand();
	}

void CBluetoothHandler::RequestFullScreen()
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	iCurrentCommand = EBemusedFullScreen;
	iPacketBuf.Copy(_L8("FULL"));
	WriteCommand();
	}

void CBluetoothHandler::RequestFindServer()
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	// If we're already connected, let's close the
	// connection to avoid confusion.
	if (iConnected)
		CloseAllResources();

	iCurrentCommand = EBemusedFindServer;
	iFindServerPort = 1;
	iPacketBuf.Copy(_L8("CHCK"));
	WriteCommand();
	}

void CBluetoothHandler::RequestSelectInPlaylist(TInt aIndex)
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	iCurrentCommand = EBemusedSelectInPlaylist;
	iPacketBuf.Copy(_L8("SLCT  "));
	iPacketBuf[4] = (TUint8) ((aIndex >> 8) & 0xFF);
	iPacketBuf[5] = (TUint8) (aIndex & 0xFF);
	WriteCommand();
	}

void CBluetoothHandler::RequestRemoveAllFromPlaylist()
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	iCurrentCommand = EBemusedRemoveAllFromPlaylist;
	iPacketBuf.Copy(_L8("RMAL"));
	WriteCommand();
	}

void CBluetoothHandler::RequestDetailedTrackInfo()
	{
	if (iCurrentCommand != EBemusedNone)
		return;

	iCurrentCommand = EBemusedDetailedTrackInfo;
	iPacketBuf.Copy(_L8("DINF"));
	WriteCommand();

//	iListener->SetDetailedInfo(44100, 128000, 2);
//	iListener->CommandComplete(iCurrentCommand, KErrNone);
	}

void CBluetoothHandler::RunL()
	{
	switch (iState)
		{
	case EReadingSongListFile:
		HandleReadFileCompleteL();
		break;

	case ESelectingServer:
		HandleSelectServerCompleteL();
		break;

	case EConnecting:
		HandleConnectionCompleteL();
		break;

	case EReading:
		HandleReadCompleteL();
		break;

	case EWriting:
		HandleWriteCompleteL();
		break;

	default:
		{
		TBuf<256> formatBuf;
		formatBuf.Format(_L("Unexpected Bluetooth state %d"), iState);
		CEikonEnv::Static()->InfoWinL(KNullDesC, formatBuf);
		}
		break;
		}
	}

void CBluetoothHandler::HandleReadFileCompleteL()
	{
	TInt ret = KErrNone;

	// If we're OK, and we got some data, parse it and get more.
	if ((iStatus.Int() >= KErrNone) && (iPacketBuf.Length() > 0))
		{
		iTotalPacketBuf.Append(iPacketBuf);
		ret = ParseListPacketL(iTotalPacketBuf);

		if (ret >= KErrNone)
			{
			iSongListFile.Read(iPacketBuf, iStatus);
			SetActive();
			return;
			}
		}

	// If we're not OK, or the parsing failed, drop out with an error.
	if ((iStatus.Int() < KErrNone) || (ret < KErrNone))
		{
		TBuf<256> resourceBuf;
		iListener->CoeEnv()->ReadResource(resourceBuf, R_READ_SONG_LIST_ERROR);

		TBuf<256> formatBuf;
		formatBuf.Format(resourceBuf, ret);
		CEikonEnv::Static()->InfoWinL(formatBuf, KNullDesC);
		}

	// We're completely finished.
	iState = EIdle;
	iSongListFile.Close();
	SortDirTree(iDirTreeRoot);
	iListener->RefreshFileListL(iDirTreeRoot);

	// If required, get info from the PC right now.
	if (iListener->Settings().iConnectToPc == TBemusedSettings::EOnStartup)
		RequestInfo();
	}

void CBluetoothHandler::HandleSelectServerCompleteL()
	{
	// Check if the user wants to use Internet sockets.
	if (iListener->Settings().iConnectUsing != TBemusedSettings::EBluetooth)
		{
		if (iStatus.Int() == KErrNone)
			{
			iNameEntry().iAddr.SetPort(7654);

			iState = EConnecting;
			iSocket.Connect(iNameEntry().iAddr, iStatus);
			SetActive();
			}
		else
			{
			ConnectionFailed(iStatus.Int());
			}
		return;
		}

	// Check for some kind of error.
	if ((iStatus.Int() != KErrNone) || (!iDeviceFoundPckg().IsValidDeviceName()))
		{
		ConnectionFailed(iStatus.Int());
		return;
		}

	// Are we checking each RFCOMM channel in turn?

	if (iQueuedCommand == EBemusedFindServer)
		{
		TInt err = KErrNone;

		if (!iProgressDialog)
			{
#ifdef BEMUSED_SERIES_60
        	TRAP(err, iProgressDialog = new (ELeave) CAknProgressDialog(
        		REINTERPRET_CAST(CEikDialog**, &iProgressDialog), ETrue));
#else
			TRAP(err, iProgressDialog = new (ELeave) CBemusedProgressDialog; );
#endif
        	}

        if (err == KErrNone)
        	{
        	iProgressDialog->PrepareLC(R_PROGRESS_NOTE);

            // Get a handle to the progress info. Must be done after PrepareLC().
            iProgressInfo = iProgressDialog->GetProgressInfoL();

            // Tell the dialog to notify us if the user presses "Cancel".
            iProgressDialog->SetCallback(this);

			TBuf<256> resourceBuf;
			iListener->CoeEnv()->ReadResource(resourceBuf, R_FINDING_BEMUSED_SERVER);

			iProgressDialog->SetTextL(resourceBuf);
#ifdef BEMUSED_SERIES_60
			iProgressDialog->SetTone(CAknNoteDialog::EConfirmationTone);
#endif

            // Action!
            iProgressDialog->RunLD();
            
			// The final value is the maximum number of RFCOMM channels minus 1.
            iProgressInfo->SetFinalValue(29);

			TBTSockAddr addr;
			addr.SetBTAddr(iDeviceFoundPckg().BDAddr());
			addr.SetPort(iFindServerPort);

			iState = EConnecting;
			iSocket.Connect(addr, iStatus);
			SetActive();
			}
		return;
		}

#ifdef BEMUSED_SERIES_60
	if (!iWaitDialog)
		{
		iWaitDialog = new (ELeave) CAknWaitDialog(
			REINTERPRET_CAST(CEikDialog**,&iWaitDialog));
		iWaitDialog->SetTone(CAknNoteDialog::EConfirmationTone);
		iWaitDialog->ExecuteLD(R_WAIT_NOTE);
		}
#else
	CEikonEnv::Static()->BusyMsgCancel();
	CEikonEnv::Static()->BusyMsgL(R_CONNECTING_MSG);
#endif

	// Do we need to force a connection on a specific channel?
	if (iListener->Settings().iBluetoothChannel > 0)
		{
		iRemotePort = iListener->Settings().iBluetoothChannel;

		TBTSockAddr addr;
		addr.SetBTAddr(iDeviceFoundPckg().BDAddr());
		addr.SetPort(iRemotePort);

		iState = EConnecting;
		iSocket.Connect(addr, iStatus);
		SetActive();
		return;
		}

	// We need to find which RFCOMM channel to use, so do an inquiry.
	if (iSdpAgent)
		{
		delete iSdpAgent;
		iSdpAgent = NULL;
		}
	iSdpAgent = CSdpAgent::NewL(*this, iDeviceFoundPckg().BDAddr());
	
	if (iSdpSearchPattern)
		{
		delete iSdpSearchPattern;
		iSdpSearchPattern = NULL;
		}

#ifdef DEBUG_SDP
	CEikonEnv::Static()->InfoWinL(_L("Starting service inquiry"), KNullDesC);
#endif

	iSdpCurrentRecordCount = 0;
	iSdpTotalRecordCount = 0;

	iSdpSearchPattern = CSdpSearchPattern::NewL();
	iSdpSearchPattern->AddL(iServiceClass);

	iSdpAgent->SetRecordFilterL(*iSdpSearchPattern);
	iSdpAgent->NextRecordRequestL();

	iState = EInquiringService;
	}

void CBluetoothHandler::HandleConnectionCompleteL()
	{
	iCurrentCommand = EBemusedNone;

	if (iStatus.Int() != KErrNone)
		{
		if (iQueuedCommand == EBemusedFindServer)
			{
//			TBuf<256> formatBuf;
//			formatBuf.Format(_L("Couldn't connect on channel %d: %d"), iFindServerPort, iStatus.Int());
//			CEikonEnv::Static()->InfoWinL(formatBuf, KNullDesC);

			FindServerNextChannel();
			}
		else
			{
			ConnectionFailed(iStatus.Int());
			}
		return;
		}

#ifdef DEBUG_SDP
	CEikonEnv::Static()->InfoWinL(_L("Connection completed successfully"), KNullDesC);
#endif
	iConnected = ETrue;

	iListener->ConnectComplete(KErrNone, iDeviceFoundPckg().BDAddr(), iRemotePort);

	if (iQueuedCommand == EBemusedList)
		{
		ShowWaitDialogL(R_REFRESHING_LIST_MSG);

		iCurrentCommand = iQueuedCommand;
		iQueuedCommand = EBemusedNone;
		WriteCommand();
		}
	else if (iQueuedCommand == EBemusedDownloadFile)
		{
		ShowWaitDialogL(R_DOWNLOADING_FILE_MSG);

		iCurrentCommand = iQueuedCommand;
		iQueuedCommand = EBemusedNone;
		WriteCommand();
		}
	else if (iQueuedCommand == EBemusedGetPlaylist)
		{
		ShowWaitDialogL(R_GETTING_PLAYLIST_MSG);

		iCurrentCommand = iQueuedCommand;
		iQueuedCommand = EBemusedNone;
		WriteCommand();
		}
	else if (iQueuedCommand == EBemusedFindServer)
		{
//		TBuf<256> formatBuf;
//		formatBuf.Format(_L("Connected on channel %d"), iFindServerPort);
//		CEikonEnv::Static()->InfoWinL(formatBuf, KNullDesC);

//		if (iWaitDialog)
//			iWaitDialog->ProcessFinishedL();

		iCommandTimer->Start(5 * 1000000, 1 * 1000000,
			TCallBack(CommandTimerCallBack, this));

		iCurrentCommand = EBemusedFindServer;
		iQueuedCommand = EBemusedNone;
		iPacketBuf.Copy(_L8("CHCK"));
		WriteCommand();
		}
	else if (iQueuedCommand != EBemusedNone)
		{
#ifdef BEMUSED_SERIES_60
		if (iWaitDialog)
			iWaitDialog->ProcessFinishedL();
#else
		CEikonEnv::Static()->BusyMsgCancel();
#endif

//		TBuf<256> formatBuf;
//		formatBuf.Format(_L("Writing %d %d"), iQueuedCommand, iPacketBuf.Length());
//		CEikonEnv::Static()->InfoWinL(formatBuf, KNullDesC);

		iCurrentCommand = iQueuedCommand;
		iQueuedCommand = EBemusedNone;
		WriteCommand();
		}
	else // No queued command, we're just connecting.
		{
#ifdef BEMUSED_SERIES_60
		if (iWaitDialog)
			iWaitDialog->ProcessFinishedL();
#else
		CEikonEnv::Static()->BusyMsgCancel();
#endif
		}
	}

void CBluetoothHandler::HandleReadCompleteL()
	{
	if (iStatus.Int() == KErrNone)
		{
		ParseCommandResponse();
		}
	else
		{
		if (iCurrentCommand == EBemusedFindServer)
			{
			FindServerNextChannel();
			return;
			}

		iListener->CommandComplete(iCurrentCommand, iStatus.Int());
		CloseAllResources();
		}
	}

void CBluetoothHandler::HandleWriteCompleteL()
	{
	if ((iStatus.Int() == KErrNone) && CommandExpectsResponse(iCurrentCommand))
		{
		iState = EReading;
		iSocket.RecvOneOrMore(iPacketBuf, 0, iStatus, iXfrLength);
		SetActive();
		}
	else
		{
		iCommandTimer->Cancel();

		if (iCurrentCommand == EBemusedFindServer)
			{
			FindServerNextChannel();
			return;
			}

		iListener->CommandComplete(iCurrentCommand, iStatus.Int());
		
		if (iStatus.Int() == KErrNone)
			{
			if (iCurrentCommand == EBemusedRemoveAllFromPlaylist)
				{
				iCurrentCommand = EBemusedNone;
				RequestGetPlaylist();
				}
			// Don't get info after a set volume command, because it isn't necessary.
			else if (iCurrentCommand != EBemusedSetVolume)
				{
				iCurrentCommand = EBemusedNone;
				RequestInfo();
				}
			else
				{
				iCurrentCommand = EBemusedNone;
				}
			}
		else
			{
			CloseAllResources();
			}
		}
	}

void CBluetoothHandler::ParseCommandResponse()
	{
	TInt err;

	switch (iCurrentCommand)
		{
	case EBemusedList:
		ParseListResponse();
		break;

	case EBemusedCurrentTrackInfo:
		ParseCurrentTrackInfoResponse();
		break;

	case EBemusedFileInfo:
		ParseFileInfoResponse();
		break;

	case EBemusedGetVersion:
		ParseGetVersionResponse();
		break;

	case EBemusedGetVolume:
		ParseGetVolumeResponse();
		break;

	case EBemusedDownloadFile:
		ParseDownloadFileResponse();
		break;

	case EBemusedFindServer:
		ParseFindServerResponse();
		break;

	case EBemusedGetPlaylist:
		TRAP(err, ParseGetPlaylistResponseL());
		break;

	case EBemusedDetailedTrackInfo:
		ParseDetailedTrackInfoResponse();
		break;

	default:
		CEikonEnv::Static()->InfoWinL(_L("Unexpected packet"), KNullDesC);
		break;
		}
	}

void CBluetoothHandler::ParseFindServerResponse()
	{
	iCommandTimer->Cancel();

	if (iPacketBuf.Find(_L8("Y")) > KErrNotFound)
		{
#ifdef BEMUSED_SERIES_60
		TRAPD(err, iProgressDialog->ProcessFinishedL());
#else
		// We don't want to be notified when the dialog is deleted
		// (otherwise we'll disconnect, etc.).
		iProgressDialog->SetCallback(NULL);

		delete iProgressDialog;
		iProgressDialog = NULL;
#endif

		TBuf<256> resourceBuf;
		iListener->CoeEnv()->ReadResource(resourceBuf, R_FOUND_BEMUSED_SERVER);

		TBuf<256> formatBuf;
		formatBuf.Format(resourceBuf, iFindServerPort);

		CEikonEnv::Static()->InfoWinL(formatBuf, KNullDesC);

		iCommandTimer->Cancel();
		iListener->CommandComplete(iCurrentCommand, iStatus.Int());
		iCurrentCommand = EBemusedNone;
		}
	else
		{
		FindServerNextChannel();
		}
	}

void CBluetoothHandler::ParseListResponse()
	{
	if (!iWaitingForResponseData)
		{
		TInt listAckIndex = iPacketBuf.Find(_L8("LISTACK"));
		if (listAckIndex > KErrNotFound)
			{
#ifdef DEBUG_SDP
			CEikonEnv::Static()->InfoWinL(_L("Found LISTACK"), KNullDesC);
#endif

			iPacketBuf.Delete(0, listAckIndex + 7);
			iTotalPacketBuf.Zero();

			iWaitingForResponseData = ETrue;

			// Open the song list file, if required.
			if (!iDirTreeNodeToExpand)
				{
				// Get the path to the song list.
				TFileName app = CEikonEnv::Static()->EikAppUi()->Application()->AppFullName();
				TParsePtr parse(app);
				TFileName songListFileName(parse.DriveAndPath());
				songListFileName.Append(KSongList);

				iSongListFile.Replace(CEikonEnv::Static()->FsSession(), songListFileName,
					EFileShareAny);
				}
			}
		else
			{
			iPacketBuf.Zero();
			iSocket.RecvOneOrMore(iPacketBuf, 0, iStatus, iXfrLength);
			SetActive();
			}
		}
	
	if (iWaitingForResponseData)
		{
		TInt ret;

		if (!iDirTreeNodeToExpand)
			{
			ret = iSongListFile.Write(iPacketBuf);
			if (ret != KErrNone)
				{
				CEikonEnv::Static()->InfoWinL(_L("Write to file failed!"), KNullDesC);
				iListener->CommandComplete(iCurrentCommand, ret);
				CloseAllResources();
				return;
				}
			}
		
//		CEikonEnv::Static()->InfoWinL(_L("Got a packet"), KNullDesC);

		iTotalPacketBuf.Append(iPacketBuf);

		ret = ParseListPacketL(iTotalPacketBuf);
		if (ret > 0)
			{
//			CEikonEnv::Static()->InfoWinL(_L("Parsed OK, getting another"), KNullDesC);

			iPacketBuf.Zero();
			iSocket.RecvOneOrMore(iPacketBuf, 0, iStatus, iXfrLength);
			SetActive();
			}
		else
			{
//			TBuf<256> formatBuf;
//			formatBuf.Format(_L("All done, ret=%d"), ret);
//			CEikonEnv::Static()->InfoWinL(formatBuf, KNullDesC);

			iSongListFile.Close();

			SortDirTree(iDirTreeRoot);

#ifdef BEMUSED_SERIES_60
			if (iWaitDialog)
				iWaitDialog->ProcessFinishedL();
#else
			CEikonEnv::Static()->BusyMsgCancel();
#endif

			iCommandTimer->Cancel();
			iListener->CommandComplete(iCurrentCommand, ret);
			iCurrentCommand = EBemusedNone;

			if (iDirTreeNodeToExpand)
				{
				// This is now an expanded directory.
				iDirTreeNodeToExpand->iFileType = KFileTypeDir;
				iListener->RefreshFileListL(iDirTreeNodeToExpand);
				}
			else
				{
				iListener->RefreshFileListL(iDirTreeRoot);
				}
			}
		}
	}

void CBluetoothHandler::ParseCurrentTrackInfoResponse()
	{
	if (!iWaitingForResponseData)
		{
		TInt infoAckIndex = iPacketBuf.Find(_L8("INFOACK"));
		if (infoAckIndex > KErrNotFound)
			{
			iPacketBuf.Delete(0, infoAckIndex + 7);
			iTotalPacketBuf.Zero();

			iWaitingForResponseData = ETrue;
			}
		else
			{
			iPacketBuf.Zero();
			iSocket.RecvOneOrMore(iPacketBuf, 0, iStatus, iXfrLength);
			SetActive();
			}
		}
	
	if (iWaitingForResponseData)
		{
		iTotalPacketBuf.Append(iPacketBuf);

		if (iTotalPacketBuf.Length() < 11)
			{
			iPacketBuf.Zero();
			iSocket.RecvOneOrMore(iPacketBuf, 0, iStatus, iXfrLength);
			SetActive();
			}
		else
			{
			TBool playing = EFalse;
			TBool paused = EFalse;
			TInt playingByte = iTotalPacketBuf[0];
			if (playingByte & 0x1)
				playing = ETrue;
			if (playingByte & 0x2)
				paused = ETrue;

			TInt length = (iTotalPacketBuf[1] << 24) + (iTotalPacketBuf[2] << 16) +
				(iTotalPacketBuf[3] << 8) + iTotalPacketBuf[4];

			TInt currentTime = (iTotalPacketBuf[5] << 24) + (iTotalPacketBuf[6] << 16) +
				(iTotalPacketBuf[7] << 8) + iTotalPacketBuf[8];

			TBool shuffle = (TBool) iTotalPacketBuf[9];
			TBool repeat = (TBool) iTotalPacketBuf[10];

			TPtrC8 guessPtr;

			if (iTotalPacketBuf.Length() > 11)
				guessPtr.Set(&iTotalPacketBuf[11], iTotalPacketBuf.Length() - 11);
			else
				guessPtr.Set(KNullDesC8);

			iListener->SetInfo(playing, paused, length, currentTime, shuffle, repeat,
				guessPtr);

			iListener->CommandComplete(iCurrentCommand, KErrNone);
			iCommandTimer->Cancel();
			iCurrentCommand = EBemusedNone;
			}
		}
	}

void CBluetoothHandler::ParseFileInfoResponse()
	{
	TBuf<256> formatBuf;
	TBuf8<256> fileNameBuf8;
	TBuf<256> resourceBuf;

	TInt finfAckIndex = iPacketBuf.Find(_L8("FINFACK"));
	if (finfAckIndex > KErrNotFound)
		{
		iCommandTimer->Cancel();

		iPacketBuf.Delete(0, finfAckIndex + 7);

		iDownloadFileLength = (iPacketBuf[0] << 24) + (iPacketBuf[1] << 16) +
			(iPacketBuf[2] << 8) + iPacketBuf[3];

		iPacketBuf.Delete(0, 4);

		TVolumeInfo volumeInfo;
		CEikonEnv::Static()->FsSession().Volume(volumeInfo,
			iListener->Settings().iDownloadDrive + EDriveC);

		// Check that there's enough space to download the file. Allow a buffer to avoid the
		// phone getting overly full.
		if (volumeInfo.iFree < (iDownloadFileLength + (128 * 1024)))
			{
			iListener->CoeEnv()->ReadResource(resourceBuf, R_NOT_ENOUGH_SPACE);
			formatBuf.Format(resourceBuf, (iDownloadFileLength / 1024) + 128);
			CEikonEnv::Static()->InfoWinL(KNullDesC, formatBuf);

			iListener->CommandComplete(iCurrentCommand, KErrTooBig);
			iCurrentCommand = EBemusedNone;
			return;
			}
        
#ifdef BEMUSED_SERIES_60
        formatBuf.Copy(KDownloadedFilesPath);
#else
		if (iListener->Settings().iDownloadDrive == 1) // drive D, ie. the memory stick
			formatBuf.Copy(KDownloadedFilesPathMs);
		else
			formatBuf.Copy(KDownloadedFilesPath);
#endif
		formatBuf[0] = 'C' + iListener->Settings().iDownloadDrive;

		TInt backslashPos = iDownloadFileName.LocateReverse('\\');
        if (backslashPos > KErrNotFound)
            {
            formatBuf.Append(_L("\\"));
            formatBuf.Append(iDownloadFileName.Mid(backslashPos + 1));
            }
        else
            {
            formatBuf.Append(_L("\\Bemused.wav"));
            }
        
		TInt err = iDownloadFile.Replace(CEikonEnv::Static()->FsSession(),
			formatBuf, EFileShareAny);

        if (err != KErrNone)
            {
            iListener->CommandComplete(iCurrentCommand, err);
			CloseAllResources();
			return;
            }
        
		iListener->CoeEnv()->ReadResource(resourceBuf, R_START_DOWNLOAD_MSG);
		formatBuf.Format(resourceBuf, &iDownloadFileName, iDownloadFileLength);
		CEikonEnv::Static()->InfoWinL(KNullDesC, formatBuf);

        if (!iProgressDialog)
        	{
#ifdef BEMUSED_SERIES_60
        	TRAP(err, iProgressDialog = new (ELeave) CAknProgressDialog(
        		REINTERPRET_CAST(CEikDialog**, &iProgressDialog), ETrue));
#else
			TRAP(err, iProgressDialog = new (ELeave) CBemusedProgressDialog; );
#endif
        	}

        if (err == KErrNone)
        	{
        	iProgressDialog->PrepareLC(R_PROGRESS_NOTE);

            // Get a handle to the progress info. Must be done after PrepareLC().
            iProgressInfo = iProgressDialog->GetProgressInfoL();

            // Tell the dialog to notify us if the user presses "Cancel".
            iProgressDialog->SetCallback(this);

            // Action!
            iProgressDialog->RunLD();
            
            iProgressInfo->SetFinalValue(iDownloadFileLength);
            }

		iCurrentCommand = EBemusedNone;

		fileNameBuf8.Copy(iDownloadFileName);
		RequestDownloadFile(fileNameBuf8);
		}
	else
		{
		iPacketBuf.Zero();
		iSocket.RecvOneOrMore(iPacketBuf, 0, iStatus, iXfrLength);
		SetActive();
		}
	}

void CBluetoothHandler::ParseDownloadFileResponse()
	{
	if (!iWaitingForResponseData)
		{
		TInt downAckIndex = iPacketBuf.Find(_L8("DOWNACK"));
		if (downAckIndex > KErrNotFound)
			{
			iPacketBuf.Delete(0, downAckIndex + 7);
			iDownloadFileReceived = 0;
			iWaitingForResponseData = ETrue;
			}
		else
			{
			iPacketBuf.Zero();
			iSocket.RecvOneOrMore(iPacketBuf, 0, iStatus, iXfrLength);
			SetActive();
			}
		}
	
	if (iWaitingForResponseData)
		{
		TInt ret;

		ret = iDownloadFile.Write(iPacketBuf);
		if (ret != KErrNone)
			{
#ifdef BEMUSED_SERIES_60
			// Get rid of the progress dialog (deletes it as well).
            iProgressDialog->ProcessFinishedL();
#else
			delete iProgressDialog;
			iProgressDialog = NULL;
#endif
            
			iListener->CommandComplete(iCurrentCommand, ret);
			CloseAllResources();
			return;
			}

		iProgressInfo->IncrementAndDraw(iPacketBuf.Length());

		iDownloadFileReceived += iPacketBuf.Length();

		if (iDownloadFileReceived >= iDownloadFileLength)
			{
#ifdef BEMUSED_SERIES_60
			// Get rid of the progress dialog (deletes it as well).
		    iProgressDialog->ProcessFinishedL();
#else
			// We don't want to be notified when the dialog is deleted
			// (otherwise we'll disconnect, etc.).
			iProgressDialog->SetCallback(NULL);

			delete iProgressDialog;
			iProgressDialog = NULL;
#endif
			// FIXME!
			CEikonEnv::Static()->InfoWinL(KNullDesC, _L("Download complete"));

			iListener->CommandComplete(iCurrentCommand, KErrNone);
			
			iDownloadFile.Close();
			iCurrentCommand = EBemusedNone;
			}
		else
			{
			iPacketBuf.Zero();
			iSocket.RecvOneOrMore(iPacketBuf, 0, iStatus, iXfrLength);
			SetActive();
			}
		}
	}

void CBluetoothHandler::ParseGetPlaylistResponseL()
	{
	if (!iWaitingForResponseData)
		{
		iPlaylistPos = -2;
		TInt playlistAckIndex = iPacketBuf.Find(_L8("PLSTACK"));
		if (playlistAckIndex > KErrNotFound)
			{
			iPacketBuf.Delete(0, playlistAckIndex + 7);
			iTotalPacketBuf.Zero();

			iWaitingForResponseData = ETrue;
			}
		else
			{
			iPacketBuf.Zero();
			iSocket.RecvOneOrMore(iPacketBuf, 0, iStatus, iXfrLength);
			SetActive();
			}
		}
	
	if (iWaitingForResponseData)
		{
		iTotalPacketBuf.Append(iPacketBuf);

		TInt ret = ParsePlaylistPacketL(iTotalPacketBuf);

		if (ret > 0)
			{
			iPacketBuf.Zero();
			iSocket.RecvOneOrMore(iPacketBuf, 0, iStatus, iXfrLength);
			SetActive();
			}
		else
			{
#ifdef BEMUSED_SERIES_60
			if (iWaitDialog)
				iWaitDialog->ProcessFinishedL();
#else
			CEikonEnv::Static()->BusyMsgCancel();
#endif

			iCommandTimer->Cancel();
			iListener->CommandComplete(iCurrentCommand, ret);
			iCurrentCommand = EBemusedNone;

			RequestInfo();
			}
		}
	}

void CBluetoothHandler::ParseDetailedTrackInfoResponse()
	{
	if (!iWaitingForResponseData)
		{
		TInt infoAckIndex = iPacketBuf.Find(_L8("DINFACK"));
		if (infoAckIndex > KErrNotFound)
			{
			iPacketBuf.Delete(0, infoAckIndex + 7);
			iTotalPacketBuf.Zero();

			iWaitingForResponseData = ETrue;
			}
		else
			{
			iPacketBuf.Zero();
			iSocket.RecvOneOrMore(iPacketBuf, 0, iStatus, iXfrLength);
			SetActive();
			}
		}
	
	if (iWaitingForResponseData)
		{
		iTotalPacketBuf.Append(iPacketBuf);

		if (iTotalPacketBuf.Length() < 12)
			{
			iPacketBuf.Zero();
			iSocket.RecvOneOrMore(iPacketBuf, 0, iStatus, iXfrLength);
			SetActive();
			}
		else
			{
			TInt bitRate;
			TInt sampleRate;
			TInt channels;

			bitRate = (iTotalPacketBuf[0] << 24) + (iTotalPacketBuf[1] << 16) +
				(iTotalPacketBuf[2] << 8) + iTotalPacketBuf[3];
			sampleRate = (iTotalPacketBuf[4] << 24) + (iTotalPacketBuf[5] << 16) +
				(iTotalPacketBuf[6] << 8) + iTotalPacketBuf[7];
			channels = (iTotalPacketBuf[8] << 24) + (iTotalPacketBuf[9] << 16) +
				(iTotalPacketBuf[10] << 8) + iTotalPacketBuf[11];

			iCommandTimer->Cancel();

			iListener->SetDetailedInfo(sampleRate, bitRate, channels);
			iListener->CommandComplete(iCurrentCommand, KErrNone);

			iCurrentCommand = EBemusedNone;
			}
		}
	}

void CBluetoothHandler::ParseGetVersionResponse()
	{
	if (!iWaitingForResponseData)
		{
		TInt versAckIndex = iPacketBuf.Find(_L8("VERSACK"));
		if (versAckIndex > KErrNotFound)
			{
			iPacketBuf.Delete(0, versAckIndex + 7);
			iTotalPacketBuf.Zero();

			iWaitingForResponseData = ETrue;
			}
		else
			{
			iPacketBuf.Zero();
			iSocket.RecvOneOrMore(iPacketBuf, 0, iStatus, iXfrLength);
			SetActive();
			}
		}
	
	if (iWaitingForResponseData)
		{
		iTotalPacketBuf.Append(iPacketBuf);

		if (iTotalPacketBuf.Length() < 2)
			{
			iPacketBuf.Zero();
			iSocket.RecvOneOrMore(iPacketBuf, 0, iStatus, iXfrLength);
			SetActive();
			}
		else
			{
			iCommandTimer->Cancel();

			iServerMajorVersion = iTotalPacketBuf[0];
			iServerMinorVersion = iTotalPacketBuf[1];

//			TBuf<32> formatBuf;
//			formatBuf.Format(_L("Maj: %d Min: %d"), iServerMajorVersion, iServerMinorVersion);
//			CEikonEnv::Static()->InfoWinL(formatBuf, KNullDesC);

			if ((iServerMajorVersion >= 1) && (iServerMinorVersion >= 73))
				{
				iCurrentCommand = EBemusedGetVolume;
				iVersionState = EGettingVolume;
				iPacketBuf.Copy(_L8("GVOL"));

				WriteCommand();
				}
			else
				{
				iVersionState = EGotVersion;

				iCurrentCommand = iQueuedCommand;
				iPacketBuf.Copy(iVersionQueuedPacketBuf);
				WriteCommand();
				}
			}
		}
	}

void CBluetoothHandler::ParseGetVolumeResponse()
	{
	if (!iWaitingForResponseData)
		{
		TInt gvolAckIndex = iPacketBuf.Find(_L8("GVOLACK"));
		if (gvolAckIndex > KErrNotFound)
			{
			iPacketBuf.Delete(0, gvolAckIndex + 7);
			iTotalPacketBuf.Zero();

			iWaitingForResponseData = ETrue;
			}
		else
			{
			gvolAckIndex = iPacketBuf.Find(_L8("GVOLNAK"));

			if (gvolAckIndex > KErrNotFound)
				{
				iCommandTimer->Cancel();

				iVersionState = EGotVolume;

				iCurrentCommand = iQueuedCommand;
				iPacketBuf.Copy(iVersionQueuedPacketBuf);
				WriteCommand();

				return;
				}
			else
				{
				iPacketBuf.Zero();
				iSocket.RecvOneOrMore(iPacketBuf, 0, iStatus, iXfrLength);
				SetActive();
				}
			}
		}
	
	if (iWaitingForResponseData)
		{
		iTotalPacketBuf.Append(iPacketBuf);

		if (iTotalPacketBuf.Length() < 1)
			{
			iPacketBuf.Zero();
			iSocket.RecvOneOrMore(iPacketBuf, 0, iStatus, iXfrLength);
			SetActive();
			}
		else
			{
			iCommandTimer->Cancel();

			iListener->ReceivedVolume(iTotalPacketBuf[0]);

			iVersionState = EGotVolume;

			iCurrentCommand = iQueuedCommand;
			iPacketBuf.Copy(iVersionQueuedPacketBuf);
			WriteCommand();
			}
		}
	}

// Returns >0 : parsed OK, need more data
// Returns  0 : parsed OK and we're finished
// Returns <0 : error!

TInt CBluetoothHandler::ParseListPacketL(TDes8& aPacket)
	{
	TChar c;
	TUint8 nodeType = 0x0;
	TUint8 fileType;
	TInt packetPos = 0;
	TInt packetLength = aPacket.Length();

#ifdef DEBUG_NODES
	TBuf<256> formatBuf;
#endif

	TPtrC8 namePtr;
	CDirTreeNode* previousNode;
	CDirTreeNode* siblingPtr;

	while (packetPos < packetLength)
		{
		c = aPacket[packetPos];
		nodeType = (TUint8) ((c >> 4) & 0xF);
		fileType = (TUint8) (c & 0xF);

#ifdef DEBUG_NODES
		formatBuf.Format(_L("nodeType: %X fileType: %X"), nodeType, fileType);
		CEikonEnv::Static()->InfoWinL(_L("First node"), formatBuf);
#endif

		namePtr.Set(ReadListFileName(aPacket, packetPos));

		// check for the "last node" token
		if (nodeType == KNodeTypeLastNode)
			return 0;

		// Note that the root node may have no filename. Otherwise, getting a blank filename
		// here means that we've run out of nodes to read.
		if ((namePtr.Length() == 0) && (nodeType != KNodeTypeRoot))
			{
#ifdef DEBUG_NODES
			formatBuf.Format(_L("Deleting %d chars of %d"), packetPos, aPacket.Length());
			CEikonEnv::Static()->InfoWinL(_L("Run out of buffer"), formatBuf);
#endif

			// we've got a truncated node in the buffer. we need to move it to the start.
			aPacket.Delete(0, packetPos);
			return 1;
			}

		previousNode = iDirTreeCurrentNode;

		switch (nodeType)
			{
		case KNodeTypeRoot: // root
#ifdef DEBUG_NODES
			CEikonEnv::Static()->InfoWinL(_L("Found root node"), KNullDesC);
#endif

			if (!iDirTreeRoot)
				{
				iDirTreeRoot = CDirTreeNode::NewL(nodeType, fileType, namePtr);
				iDirTreeCurrentNode = iDirTreeRoot;
				iDirTreeCurrentNode->iFirstChild = NULL;
				}
			else
				{
#ifdef DEBUG_NODES
				CEikonEnv::Static()->InfoWinL(_L("...but already have root!"), KNullDesC);
#endif
				// Do nothing; the remote end may think that this node's the root when we know
				// that it isn't.
//				return KErrCorrupt;
				}
			break;

		case KNodeTypeChild: // child
			if (previousNode)
				{
#ifdef DEBUG_NODES
				formatBuf.Copy(*previousNode->iFileName);
				CEikonEnv::Static()->InfoWinL(_L("Found child of"), formatBuf);
#endif

				iDirTreeCurrentNode = CDirTreeNode::NewL(nodeType, fileType, namePtr);
				iDirTreeCurrentNode->iParent = previousNode;
				iDirTreeCurrentNode->iFirstChild = NULL;

				if (previousNode->iFirstChild)
					{
#ifdef DEBUG_NODES
					formatBuf.Copy(*previousNode->iFirstChild->iFileName);
					CEikonEnv::Static()->InfoWinL(_L("...it's a sibling of"), formatBuf);
#endif

					// find the last of the parent's children
					siblingPtr = previousNode->iFirstChild;
					while (siblingPtr->iNextSibling)
						siblingPtr = siblingPtr->iNextSibling;

					siblingPtr->iNextSibling = iDirTreeCurrentNode;
					}
				else
					{
#ifdef DEBUG_NODES
					CEikonEnv::Static()->InfoWinL(_L("...it's the first child"), KNullDesC);
#endif
					previousNode->iFirstChild = iDirTreeCurrentNode;
					}
				}
			else
				{
#ifdef DEBUG_NODES
				CEikonEnv::Static()->InfoWinL(_L("Child node found, but no previous node!"),
					KNullDesC);
#endif
				return KErrCorrupt;
				}
			break;

		case KNodeTypeOnlyChild: // only child
			if (previousNode)
				{
#ifdef DEBUG_NODES
				formatBuf.Copy(*previousNode->iFileName);
				CEikonEnv::Static()->InfoWinL(_L("Found only child of"), formatBuf);
#endif

				iDirTreeCurrentNode = CDirTreeNode::NewL(nodeType, fileType, namePtr);
				iDirTreeCurrentNode->iParent = previousNode;

				previousNode->iFirstChild = iDirTreeCurrentNode;
				}
			else
				{
#ifdef DEBUG_NODES
				CEikonEnv::Static()->InfoWinL(_L("Found only child, but no previous node!"),
					KNullDesC);
#endif
				return KErrCorrupt;
				}
			break;

		case KNodeTypeSibling: // sibling
			// If we know there are no more siblings on this level, go up until there are.
			while (previousNode &&
				((previousNode->iNodeType == KNodeTypeOnlyChild) ||
				(previousNode->iNodeType == KNodeTypeLastSibling)))
				{
				previousNode = previousNode->iParent;
				}

			if (previousNode)
				{
#ifdef DEBUG_NODES
				formatBuf.Copy(*previousNode->iFileName);
				CEikonEnv::Static()->InfoWinL(_L("Found sibling of"), formatBuf);
#endif

				iDirTreeCurrentNode = CDirTreeNode::NewL(nodeType, fileType, namePtr);
				iDirTreeCurrentNode->iParent = previousNode->iParent;

				previousNode->iNextSibling = iDirTreeCurrentNode;
				}
			else
				{
#ifdef DEBUG_NODES
				CEikonEnv::Static()->InfoWinL(_L("Found sibling, but no previous node!"), KNullDesC);
#endif
				return KErrCorrupt;
				}
			break;

		case KNodeTypeLastSibling: // last sibling
			// If we know there are no more siblings on this level, go up until there are.
			while (previousNode &&
				((previousNode->iNodeType == KNodeTypeOnlyChild) ||
				(previousNode->iNodeType == KNodeTypeLastSibling)))
				{
				previousNode = previousNode->iParent;
				}

			if (previousNode)
				{
#ifdef DEBUG_NODES
				formatBuf.Copy(*previousNode->iFileName);
				CEikonEnv::Static()->InfoWinL(_L("Found last sibling of"), formatBuf);
#endif

				iDirTreeCurrentNode = CDirTreeNode::NewL(nodeType, fileType, namePtr);
				iDirTreeCurrentNode->iParent = previousNode->iParent;

				previousNode->iNextSibling = iDirTreeCurrentNode;
				}
			else
				{
#ifdef DEBUG_NODES
				CEikonEnv::Static()->InfoWinL(_L("Found last sibling, but no previous node!"),
					KNullDesC);
#endif
				return KErrCorrupt;
				}
			break;

		default:
#ifdef DEBUG_NODES
			formatBuf.Format(_L("nodeType: %X fileType: %X"), nodeType, fileType);
			CEikonEnv::Static()->InfoWinL(_L("Found unknown element"), formatBuf);
#endif
			return KErrCorrupt;
			break;
			}
		}

	// we've read everything we can, but we still need more.
	aPacket.Zero();

	return 1;
	}

// Returns >0 : parsed OK, need more data
// Returns  0 : parsed OK and we're finished
// Returns <0 : error!
TInt CBluetoothHandler::ParsePlaylistPacketL(TDes8& aPacket)
	{
	// First, check the first couple of bytes to get the currently selected
	// playlist item.
	if (iPlaylistPos < -1)
		{
		if (aPacket.Length() >= 2)
			{
			iPlaylistPos = (aPacket[0] << 8) + aPacket[1];
			iListener->SetPlaylistIndex(iPlaylistPos);
			aPacket.Delete(0, 2);
			}
		else
			{
			return 1;
			}
		}

	TInt newlinePos;

	while ((newlinePos = aPacket.Locate('\n')) >= 0)
		{
		if (newlinePos == 0)
			{
			aPacket.Delete(0, newlinePos + 1);
			continue;
			}

		TPtrC8 linePtr = aPacket.Left(newlinePos);

		if (linePtr[newlinePos-1] == '\r')
			linePtr.Set(aPacket.Left(newlinePos-1));

		// Exclude additional "info" lines.
		if (linePtr[0] != '#')
			{
			// Just get the filename.

			if (linePtr.Locate('\\') != KErrNotFound)
				linePtr.Set(linePtr.Mid(linePtr.LocateReverse('\\') + 1));

			if (linePtr.FindF(_L8(".mp3")) != KErrNotFound)
				linePtr.Set(linePtr.Left(linePtr.FindF(_L8(".mp3"))));

//				TBuf<256> infoBuf;
//				infoBuf.Copy(linePtr);
//				CEikonEnv::Static()->InfoWinL(_L("Playlist file:"), infoBuf);

			HBufC8* lineBuf = linePtr.AllocL();
			iListener->Playlist().Append(lineBuf);
			}
//			else
//				{
//				TBuf<256> infoBuf;
//				infoBuf.Copy(linePtr);
//				CEikonEnv::Static()->InfoWinL(_L("Ignored line:"), infoBuf);
//				}

		aPacket.Delete(0, newlinePos + 1);
		}

	// Have we got everything?
	if ((iPlaylistPos >= -1) && (aPacket.Locate('\0') != KErrNotFound))
		{
		aPacket.Zero();
		return 0;
		}

	return 1;
	}

TPtrC8 CBluetoothHandler::ReadListFileName(const TDesC8& aPacket, TInt& aStartPos)
	{
	TInt startPos = aStartPos + 1;
	TInt endPos = aStartPos + 1;
	TInt packetLength = aPacket.Length();

	while ((endPos < packetLength) && (aPacket[endPos] != '\0'))
		endPos++;

	if (endPos < packetLength)
		{
		aStartPos = endPos + 1;
		return aPacket.Mid(startPos, endPos - startPos);
		}
	return KNullDesC8();
	}

void CBluetoothHandler::SortDirTree(CDirTreeNode* aDirectory)
	{
	if (!aDirectory || !aDirectory->iFirstChild)
		return;

	// Put the files in this directory into an array.

	RPointerArray<CDirTreeNode> sortArray;
	TLinearOrder<CDirTreeNode> order(CompareNodes);
	CDirTreeNode* searchNode = aDirectory->iFirstChild;

	while (searchNode)
		{
		sortArray.Append(searchNode);
		searchNode = searchNode->iNextSibling;
		}

	// Action!

	sortArray.Sort(order);

	// Put the files back into the tree.

	aDirectory->iFirstChild = sortArray[0];
	aDirectory->iFirstChild->iNodeType = KNodeTypeChild;
	searchNode = aDirectory->iFirstChild;

	TInt count = sortArray.Count();
	for (TInt i = 1; i < count; i++)
		{
		searchNode->iNextSibling = sortArray[i];
		searchNode->iNextSibling->iNodeType = KNodeTypeSibling;
		searchNode = searchNode->iNextSibling;
		}
	searchNode->iNodeType = KNodeTypeLastSibling;
	searchNode->iNextSibling = NULL;

	sortArray.Reset();

	// Now sort the children.

	searchNode = aDirectory->iFirstChild;
	while (searchNode)
		{
		SortDirTree(searchNode);
		searchNode = searchNode->iNextSibling;
		}
	}

TInt CBluetoothHandler::CompareNodes(const CDirTreeNode& aNode1, const CDirTreeNode& aNode2)
	{
	// This comparison function places directories before files, and does an alphabetical
	// ordering on files of the same type.
	
	if (((aNode1.iFileType == KFileTypeDir) || (aNode1.iFileType == KFileTypeUnexpandedDir)) &&
		(aNode2.iFileType == KFileTypeFile))
		{
		return -1;
		}

	if (((aNode2.iFileType == KFileTypeDir) || (aNode2.iFileType == KFileTypeUnexpandedDir)) &&
		(aNode1.iFileType == KFileTypeFile))
		{
		return 1;
		}

	return aNode1.iFileName->CompareF(*aNode2.iFileName);
	}

void CBluetoothHandler::DoCancel()
	{
	switch (iState)
		{
	case ESelectingServer:
		iNotifier.CancelNotifier(KDeviceSelectionNotifierUid);
		break;
	case EInquiringService:
		break;
	case EConnecting:
		iSocket.CancelConnect();
		break;
	case EReading:
		iSocket.CancelRead();
		break;
	case EWriting:
		iSocket.CancelWrite();
		break;
	default:
		break;
		}
	}

TInt CBluetoothHandler::RunError(TInt aError)
	{
	TBuf<256> formatBuf;
	if (aError != KErrNone)
		{
		formatBuf.Format(_L("Unexpected error %d in state %d"), aError, iState);
		CEikonEnv::Static()->InfoWinL(formatBuf, KNullDesC);
		}
	return KErrNone;
	}

TBool CBluetoothHandler::CommandExpectsResponse(TBemusedCommand aCommand)
	{
	if ((aCommand == EBemusedList) || (aCommand == EBemusedCurrentTrackInfo) ||
		(aCommand == EBemusedFileInfo) || (aCommand == EBemusedDownloadFile) ||
		(aCommand == EBemusedFindServer) || (aCommand == EBemusedGetPlaylist) ||
		(aCommand == EBemusedDetailedTrackInfo) || (aCommand == EBemusedGetVersion) ||
		(aCommand == EBemusedGetVolume))
		{
		return ETrue;
		}
	else
		{
		return EFalse;
		}
	}

void CBluetoothHandler::AttributeRequestComplete(TSdpServRecordHandle /*aHandle*/, TInt aError)
	{
	if (aError != KErrNone)
		{
		ConnectionFailed(aError);
		return;
		}

	iSdpCurrentRecordCount++;

	if (iSdpCurrentRecordCount >= iSdpTotalRecordCount)
		{
#ifdef DEBUG_SDP
		CEikonEnv::Static()->InfoWinL(_L("Got all SDP records"), KNullDesC);
#endif

		TBTSockAddr addr;
		addr.SetBTAddr(iDeviceFoundPckg().BDAddr());
		addr.SetPort(iRemotePort);

		iState = EConnecting;
		iSocket.Connect(addr, iStatus);
		SetActive();
		}
	else
		{
#ifdef DEBUG_SDP
		CEikonEnv::Static()->InfoWinL(_L("Making another SDP request"), KNullDesC);
#endif
		iSdpAgent->NextRecordRequestL();
		}
	}

void CBluetoothHandler::AttributeRequestResult(TSdpServRecordHandle /*aHandle*/,
											   TSdpAttributeID aAttrID,
											   CSdpAttrValue* aAttrValue)
	{
#ifdef DEBUG_SDP
	TBuf<256> formatBuf;
	formatBuf.Format(_L("attr id: %d type: %d"), aAttrID, aAttrValue->Type());
	CEikonEnv::Static()->InfoWinL(_L("Got attribute"), formatBuf);
#else
	(void) aAttrID;
	(void) aAttrValue;
#endif
	
	if (aAttrValue->Type() == ETypeDES)
		{
		CSdpAttrValueDES* attrValueDES = (CSdpAttrValueDES*) aAttrValue;
		TRAPD(err, attrValueDES->AcceptVisitorL(*this));
		}

	delete aAttrValue; // avoid memory leak
	}

void CBluetoothHandler::NextRecordRequestComplete(TInt aError, TSdpServRecordHandle aHandle,
		TInt aTotalRecordsCount)
	{
#ifdef DEBUG_SDP
	TBuf<256> formatBuf;
	formatBuf.Format(_L("err %d count %d"), aError, aTotalRecordsCount);
	CEikonEnv::Static()->InfoWinL(_L("Next record req complete"), formatBuf);
#endif

	if (aError != KErrNone)
		{
		ConnectionFailed(aError);
		}
	else
		{
		iSdpTotalRecordCount = aTotalRecordsCount;
		iSdpAgent->AttributeRequestL(aHandle, KSdpAttrIdProtocolDescriptorList);
		}
	}

void CBluetoothHandler::VisitAttributeValueL(CSdpAttrValue &aValue, TSdpElementType aType)
	{
#ifdef DEBUG_SDP
	TBuf<256> formatBuf;
	formatBuf.Format(_L("type: %d"), aType);
	CEikonEnv::Static()->InfoWinL(_L("Visited"), formatBuf);
#else
	(void) aType;
#endif

	if (aValue.Type() == ETypeUUID)
		{
#ifdef DEBUG_SDP
		TUUID uuid = aValue.UUID();
		TBuf<256> uuidBuf;
		uuidBuf.Copy(uuid.ShortestForm());
		uuidBuf.SetMax();

		formatBuf.Format(_L("%02X.%02X.%02X.%02X"), uuidBuf[0], uuidBuf[1],
			uuidBuf[2], uuidBuf[3]);
		CEikonEnv::Static()->InfoWinL(_L("UUID"), formatBuf);
#endif
		}
	else if (aValue.Type() == ETypeUint)
		{
		iRemotePort = aValue.Uint();

#ifdef DEBUG_SDP
		formatBuf.Format(_L("%d"), iRemotePort);
		CEikonEnv::Static()->InfoWinL(_L("Unsigned int"), formatBuf);
#endif
		}
	}

void CBluetoothHandler::StartListL(CSdpAttrValueList& /*aList*/)
	{
	}

void CBluetoothHandler::EndListL()
	{
	}

void CBluetoothHandler::ConnectionFailed(TInt aError)
	{
#ifdef DEBUG_SDP
	TBuf<256> formatBuf;
	formatBuf.Format(_L("aError=%d"), aError);
	CEikonEnv::Static()->InfoWinL(_L("CBluetoothHandler::ConnectionFailed"), formatBuf);
#endif

	if (iQueuedCommand != EBemusedNone)
		{
		iListener->CommandComplete(iQueuedCommand, aError);
		iQueuedCommand = EBemusedNone;
		}
	else
		{
		iListener->ConnectComplete(aError, iDeviceFoundPckg().BDAddr(), iRemotePort);
		}
	CloseAllResources();
	}

void CBluetoothHandler::DialogDismissedL(TInt /*aButtonId*/)
	{
	// Called when a progress or wait dialog is dismissed. We always want to cancel
	// the current operation if this happens.

	iListener->CommandComplete(iCurrentCommand, KErrCancel);

	Cancel();

	iState = EIdle;
	iConnected = EFalse;
	iCurrentCommand = EBemusedNone;
	iQueuedCommand = EBemusedNone;

	iCommandTimer->Cancel();

	iSongListFile.Close();
	iDownloadFile.Close();
	iSocket.Close();

	// For UIQ, we have to set this to NULL.
	iProgressDialog = NULL;
	}

TInt CBluetoothHandler::CommandTimerCallBack(TAny* aBluetoothHandler)
	{
	((CBluetoothHandler*)aBluetoothHandler)->CommandTimedOut();
	return 0;
	}

void CBluetoothHandler::CommandTimedOut()
	{
	Cancel();

	if (iCurrentCommand == EBemusedFindServer)
		{
		FindServerNextChannel();
		}
	else if (iCurrentCommand == EBemusedGetVersion)
		{
		iVersionState = EGotVersion;

		iCurrentCommand = iQueuedCommand;
		iPacketBuf.Copy(iVersionQueuedPacketBuf);
		WriteCommand();
		}
	else
		{
		iCommandTimer->Cancel();
		iListener->CommandComplete(iCurrentCommand, KErrTimedOut);
		CloseAllResources();
		}
	}

void CBluetoothHandler::FindServerNextChannel()
	{
	iCommandTimer->Cancel();

	iSocket.Close();
	iSocket.Open(iSocketServ, KBTAddrFamily, KSockStream, KRFCOMM);

	iFindServerPort++;

	if (iFindServerPort > 30)
		{
		CloseAllResources();

		TBuf<256> resourceBuf;
		iListener->CoeEnv()->ReadResource(resourceBuf, R_FIND_BEMUSED_SERVER_ERROR);
		CEikonEnv::Static()->InfoWinL(resourceBuf, KNullDesC);
		iListener->CommandComplete(EBemusedFindServer, KErrNone);
		return;
		}

	if (iProgressInfo)
		iProgressInfo->IncrementAndDraw(1);

//	TBuf<256> formatBuf;
//	formatBuf.Format(_L("Trying channel %d"), iFindServerPort);
//	CEikonEnv::Static()->InfoWinL(formatBuf, KNullDesC);

	TBTSockAddr addr;
	addr.SetBTAddr(iDeviceFoundPckg().BDAddr());
	addr.SetPort(iFindServerPort);

	iState = EConnecting;
	iQueuedCommand = EBemusedFindServer;
	iSocket.Connect(addr, iStatus);
	SetActive();
	}

void CBluetoothHandler::ShowWaitDialogL(TInt aResourceId)
	{
#ifdef BEMUSED_SERIES_60
	TBuf<256> resourceBuf;
	iListener->CoeEnv()->ReadResource(resourceBuf, aResourceId);

	if (!iWaitDialog)
		{
		iWaitDialog = new (ELeave) CAknWaitDialog(
			REINTERPRET_CAST(CEikDialog**,&iWaitDialog));
		iWaitDialog->SetTone(CAknNoteDialog::EConfirmationTone);
		iWaitDialog->SetTextL(resourceBuf);
		iWaitDialog->ExecuteLD(R_WAIT_NOTE);
		}
	else
		{
		iWaitDialog->SetTextL(resourceBuf);
		}
#else
	CEikonEnv::Static()->BusyMsgCancel();
	CEikonEnv::Static()->BusyMsgL(aResourceId);
#endif
	}