/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#include "BemusedBehaviourSettingsContainer.h"
#include "BemusedBehaviourSettingsView.h"
#include "BemusedApp.h"
#include "BemusedAppUi.h"
#include "Bemused.hrh"
#include <bemused.rsg>

#include <eikfutil.h>
#include <gdi.h>
#include <barsread.h>
#include <aknsettingitemlist.h>

////////////////////////////////////////////////////////////////////////////////
// Helper class providing the setting item list.
////////////////////////////////////////////////////////////////////////////////

CBemusedBehaviourSettingItemList::CBemusedBehaviourSettingItemList(TBemusedSettings& aData)
	: iData(aData)
	{
	}

CAknSettingItem* CBemusedBehaviourSettingItemList::CreateSettingItemL(TInt aSettingId)
	{
	switch (aSettingId)
		{
	case EBemusedActionOnIncomingCallId:
		return new (ELeave) CAknEnumeratedTextPopupSettingItem(
            aSettingId, iData.iActionOnIncomingCall);

	case EBemusedActionOnStoppingId:
		return new (ELeave) CAknEnumeratedTextPopupSettingItem(
            aSettingId, iData.iActionOnStopping);

	case EBemusedRetrieveListId:
		return new (ELeave) CAknEnumeratedTextPopupSettingItem(
            aSettingId, iData.iRetrieveList);

	case EBemusedConnectToPcId:
		return new (ELeave) CAknEnumeratedTextPopupSettingItem(
            aSettingId, iData.iConnectToPc);

	case EBemusedBluetoothChannelId:
		return new (ELeave) CAknEnumeratedTextPopupSettingItem(
			aSettingId, iData.iBluetoothChannel);

	case EBemusedDownloadDriveId:
		return new (ELeave)
			CAknEnumeratedTextPopupSettingItem(aSettingId, iData.iDownloadDrive);

	case EBemusedChangeViewId:
		return new (ELeave) CAknEnumeratedTextPopupSettingItem(
			aSettingId, iData.iChangeView);

	case EBemusedSelectingItemId:
		return new (ELeave) CAknEnumeratedTextPopupSettingItem(
			aSettingId, iData.iSelectingItem);

	case EBemusedConnectUsingId:
		return new (ELeave) CAknEnumeratedTextPopupSettingItem(
			aSettingId, iData.iConnectUsing);

	case EBemusedServerAddressId:
		return new (ELeave) CAknTextSettingItem(aSettingId, iData.iServerAddress);

	default:
		break;
		}
	return NULL;
	}

void CBemusedBehaviourSettingItemList::PopulateDriveLettersL(CAknEnumeratedTextPopupSettingItem* aSettingItem)
	{
	TDriveList driveList;
	User::LeaveIfError(CEikonEnv::Static()->FsSession().DriveList(driveList));

	CArrayPtrFlat<CAknEnumeratedText>* enumTextArray =
		new (ELeave) CArrayPtrFlat<CAknEnumeratedText>(4);
	CleanupStack::PushL(enumTextArray);

	CArrayPtrFlat<HBufC>* poppedUpTextArray = new (ELeave) CArrayPtrFlat<HBufC>(4);
	CleanupStack::PushL(poppedUpTextArray);

	TBuf<1> driveLetter;
	driveLetter.SetMax();
	TInt count = 0;
	for (TInt i = 0; i < 25; i++)
		{
		if (driveList[i])
			{
			driveLetter[0] = 'A' + i;
			poppedUpTextArray->AppendL(driveLetter.AllocL());
			enumTextArray->AppendL(new (ELeave) CAknEnumeratedText
				(count, driveLetter.AllocL()));
			count++;
			}
		}

	aSettingItem->SetEnumeratedTextArrays(enumTextArray, poppedUpTextArray);
	CleanupStack::Pop(2); // enumTextArray, poppedUpTextArray

	aSettingItem->HandleTextArrayUpdateL();
	}


///////////////////////////////////////////////////////////////////////////////
// The settings container.
////////////////////////////////////////////////////////////////////////////////

void CBemusedBehaviourSettingsContainer::ConstructL(CBemusedAppUi* aAppUi,
													CBemusedBehaviourSettingsView* aView,
													const TRect& aRect)
    {
	iAppUi = aAppUi;
	iView = aView;

    CreateWindowL();

    SetRect(aRect);
    ActivateL();

	iListBox = new (ELeave) CBemusedBehaviourSettingItemList(iAppUi->Settings());
    iListBox->SetMopParent(this);
    iListBox->ConstructFromResourceL(R_BEMUSED_BEHAVIOUR_SETTING_ITEM_LIST);

	// We have to call this after the list box has been constructed from the resource.

	iListBox->PopulateDriveLettersL((CAknEnumeratedTextPopupSettingItem*)
		(*iListBox->SettingItemArray())[KDownloadDriveSettingIndex]);

	iListBox->LoadSettingsL();

    iListBox->SetRect(aRect);
    iListBox->ActivateL();
    }

CBemusedBehaviourSettingsContainer::~CBemusedBehaviourSettingsContainer()
    {
	if (iListBox)
		{
		TRAPD(ignore, iListBox->StoreSettingsL());
		}

	delete iListBox;
    }

void CBemusedBehaviourSettingsContainer::SizeChanged()
    {
	if (iListBox)
		iListBox->SetRect(Rect());
    }

TInt CBemusedBehaviourSettingsContainer::CountComponentControls() const
    {
	if (iListBox)
		return 1;
	else
		return 0;
    }

CCoeControl* CBemusedBehaviourSettingsContainer::ComponentControl(TInt aIndex) const
    {
    switch (aIndex)
        {
        case 0:
			return iListBox;
        default:
            return NULL;
        }
    }

void CBemusedBehaviourSettingsContainer::Draw(const TRect& aRect) const
    {
    CWindowGc& gc = SystemGc();

    gc.SetPenStyle(CGraphicsContext::ENullPen);
    gc.SetBrushColor(KRgbGray);
    gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
    gc.DrawRect(aRect);
    }

void CBemusedBehaviourSettingsContainer::HandleControlEventL(CCoeControl* /*aControl*/,
													TCoeEvent /*aEventType*/)
    {
    }

TKeyResponse CBemusedBehaviourSettingsContainer::OfferKeyEventL(const TKeyEvent& aKeyEvent,
													   TEventCode aType)
    {
	return iListBox->OfferKeyEventL(aKeyEvent, aType);
	}

// End of File  
