/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#include <aknviewappui.h>
#include <avkon.hrh>
#include <Bemused.rsg>
#include "BemusedBrowseView.h"
#include "BemusedAppUi.h"
#include "BemusedBrowseContainer.h" 

void CBemusedBrowseView::ConstructL()
    {
    BaseConstructL(R_BEMUSED_BROWSE_VIEW);

	iTextForegroundColour = KRgbBlack;
	iTextBackgroundColour = KRgbWhite;
	iHighlightForegroundColour = KRgbBlack;
	iHighlightBackgroundColour = TRgb(0x80, 0x80, 0xFF);
    }

CBemusedBrowseView::~CBemusedBrowseView()
    {
    if (iContainer)
        AppUi()->RemoveFromViewStack(*this, iContainer);

    delete iContainer;
    }

TUid CBemusedBrowseView::Id() const
    {
    return KBrowseViewId;
    }

void CBemusedBrowseView::HandleCommandL(TInt aCommand)
    {   
    switch (aCommand)
        {
        case EAknSoftkeyOk:
            break;

        case EAknSoftkeyExit:
            AppUi()->HandleCommandL(EEikCmdExit);
            break;

        default:
            AppUi()->HandleCommandL(aCommand);
            break;
        }
    }

void CBemusedBrowseView::HandleClientRectChange()
    {
    if (iContainer)
        iContainer->SetRect(ClientRect());
    }

void CBemusedBrowseView::DoActivateL(const TVwsViewId& /*aPrevViewId*/, TUid /*aCustomMessageId*/,
	const TDesC8& /*aCustomMessage*/)
    {
    if (!iContainer)
        {
        iContainer = new (ELeave) CBemusedBrowseContainer;
        iContainer->SetMopParent(this);

		CBemusedAppUi* appUi = (CBemusedAppUi*) AppUi();

		iContainer->SetColours(iTextForegroundColour, iTextBackgroundColour,
			iHighlightForegroundColour, iHighlightBackgroundColour);

		// Set the container's dimensions based on whether we're full-screen or not.
		if (appUi->Settings().iDisplayType == TBemusedSettings::EFullScreen)
			iContainer->ConstructL(appUi, TRect(0, 0, KFullScreenWidth, KFullScreenHeight));
		else
			iContainer->ConstructL(appUi, ClientRect());

		TBuf<256> resourceBuf;
		
		if (iListLoaded)
			appUi->CoeEnv()->ReadResource(resourceBuf, R_EMPTY_SONG_LIST);
		else
			appUi->CoeEnv()->ReadResource(resourceBuf, R_LOADING_SONG_LIST);

		iContainer->SetListEmptyStringL(resourceBuf);

        AppUi()->AddToStackL(*this, iContainer);

		iContainer->DrawNow();
		} 
	}

void CBemusedBrowseView::DoDeactivate()
    {
    if (iContainer)
        AppUi()->RemoveFromViewStack(*this, iContainer);
    
    delete iContainer;
    iContainer = NULL;
    }

void CBemusedBrowseView::ReadSkinDescriptionL(const TDesC& aFileName)
	{
	// This function reads the bits of the skin description which this
	// view is interested in.

	// Set default values in case we can't open the skin.
	iTextForegroundColour = KRgbBlack;
	iTextBackgroundColour = KRgbWhite;
	iHighlightForegroundColour = KRgbBlack;
	iHighlightBackgroundColour = TRgb(0x80, 0x80, 0xFF);

	RFile file;
	User::LeaveIfError(file.Open(CEikonEnv::Static()->FsSession(),
		aFileName, EFileShareAny | EFileRead));
	CleanupClosePushL(file);

	TInt fileSize;
	User::LeaveIfError(file.Size(fileSize));

	HBufC8* fileBuf = HBufC8::NewLC(fileSize);
	TPtr8 fileBufPtr = fileBuf->Des();

	User::LeaveIfError(file.Read(fileBufPtr));

	TLex8 lex(*fileBuf);
	TPtrC8 ptr(lex.NextToken());

	while (ptr != KNullDesC8)
		{
		lex.SkipSpace();

		if (!ptr.CompareF(_L8("textforegroundcolour:")))
			ReadSkinColour(lex, iTextForegroundColour);
		else if (!ptr.CompareF(_L8("textbackgroundcolour:")))
			ReadSkinColour(lex, iTextBackgroundColour);
		else if (!ptr.CompareF(_L8("highlightforegroundcolour:")))
			ReadSkinColour(lex, iHighlightForegroundColour);
		else if (!ptr.CompareF(_L8("highlightbackgroundcolour:")))
			ReadSkinColour(lex, iHighlightBackgroundColour);
		else
			lex.SkipSpace();

		ptr.Set(lex.NextToken());
		}

	CleanupStack::PopAndDestroy(2); // file, fileBuf
	}

void CBemusedBrowseView::ReadSkinColour(TLex8& aLex, TRgb& aColour)
	{
	TInt colour;

	aLex.Val(colour);
	aColour.SetRed(colour);
	aLex.SkipSpace();

	aLex.Val(colour);
	aColour.SetGreen(colour);
	aLex.SkipSpace();

	aLex.Val(colour);
	aColour.SetBlue(colour);
	aLex.SkipSpace();
	}

CBemusedBrowseContainer* CBemusedBrowseView::BrowseContainer()
	{
	return iContainer;
	}

void CBemusedBrowseView::SetListLoaded(TBool aListLoaded)
	{
	if (iContainer && !iListLoaded && aListLoaded)
		{
		TBuf<256> resourceBuf;
		((CBemusedAppUi*)AppUi())->CoeEnv()->ReadResource(resourceBuf, R_EMPTY_SONG_LIST);
		iContainer->SetListEmptyStringL(resourceBuf);
		}

	iListLoaded = aListLoaded;
	}

// End of File

