/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#include "BemusedDisplaySettingsContainer.h"
#include "BemusedDisplaySettingsView.h"
#include "BemusedApp.h"
#include "BemusedAppUi.h"
#include "Bemused.hrh"
#include <bemused.rsg>

#include <eikfutil.h>
#include <gdi.h>
#include <barsread.h>
#include <aknsettingitemlist.h>
#include <msvapi.h> // Defines MessageServer::CurrentDriveL()

////////////////////////////////////////////////////////////////////////////////
// Helper class providing the setting item list.
////////////////////////////////////////////////////////////////////////////////

CBemusedDisplaySettingItemList::CBemusedDisplaySettingItemList(TBemusedSettings& aData)
	: iData(aData)
	{
	}

CAknSettingItem* CBemusedDisplaySettingItemList::CreateSettingItemL(TInt aSettingId)
	{
	switch (aSettingId)
		{
	case EBemusedBrowserFontId:
		return new (ELeave) CAknEnumeratedTextPopupSettingItem(
            aSettingId, iData.iSongBrowserFont);

	case EBemusedDisplayTypeId:
		return new (ELeave) CAknEnumeratedTextPopupSettingItem(
            aSettingId, iData.iDisplayType);

	case EBemusedDisplayFilesId:
		return new (ELeave) CAknEnumeratedTextPopupSettingItem(
            aSettingId, iData.iDisplayFiles);

	case EBemusedTimeDisplayId:
		return new (ELeave) CAknEnumeratedTextPopupSettingItem(
            aSettingId, iData.iTimeDisplay);

	case EBemusedSkinFileNameId:
		return new (ELeave) CAknEnumeratedTextPopupSettingItem(
			aSettingId, iData.iSkinFileIndex);

	case EBemusedInstallSkinId:
		return new (ELeave) CAknEnumeratedTextPopupSettingItem(
			aSettingId, iData.iInstallSkinIndex);

	case EBemusedUninstallSkinId:
		return new (ELeave) CAknEnumeratedTextPopupSettingItem(
			aSettingId, iData.iUninstallSkinIndex);

	case EBemusedUseId3TagsId:
		return new (ELeave) CAknEnumeratedTextPopupSettingItem(
			aSettingId, iData.iUseId3Tags);

	default:
		break;
		}
	return NULL;
	}


///////////////////////////////////////////////////////////////////////////////
// The settings container.
////////////////////////////////////////////////////////////////////////////////

void CBemusedDisplaySettingsContainer::GetSkinListL(CAknEnumeratedTextPopupSettingItem* aSettingItem,
											 TBool aAddSelectItem)
	{
	// Gets the list of skins in the Bemused skins directory.

	CDir* entryList;
	TFileName skinPath;
	TInt ret;

	if (iAppUi->Settings().iDisplayType == TBemusedSettings::EInWindow)
		skinPath.Format(_L("%S\\%S"), &iAppUi->SkinPathL(), &KMatchInWindowSkinBitmaps);
	else
		skinPath.Format(_L("%S\\%S"), &iAppUi->SkinPathL(), &KMatchFullScreenSkinBitmaps);

	// Get a list of entries in the skins directory matching the criteria.

	ret = CEikonEnv::Static()->FsSession().GetDir(skinPath, KEntryAttNormal,
		ESortByName, entryList);

	if (ret != KErrNone)
		{
		delete entryList;

		TBuf<256> formatBuf;
		formatBuf.Format(_L("Couldn't get list of skins: %d"), ret);
		CEikonEnv::Static()->InfoWinL(formatBuf, KNullDesC);
		return;
		}

	TBuf<256> formatBuf;
	TBuf<256> resourceBuf;
	CEikonEnv::Static()->ReadResource(resourceBuf, R_SELECT_SKIN_MSG);

	CleanupStack::PushL(entryList);

	CArrayPtrFlat<CAknEnumeratedText>* enumTextArray =
		new (ELeave) CArrayPtrFlat<CAknEnumeratedText>(4);
	CleanupStack::PushL(enumTextArray);

	CArrayPtrFlat<HBufC>* poppedUpTextArray = new (ELeave) CArrayPtrFlat<HBufC>(4);
	CleanupStack::PushL(poppedUpTextArray);

	// Add a "select" entry to the list of items if necessary.

	if (aAddSelectItem)
		{
		poppedUpTextArray->AppendL(resourceBuf.AllocL());
		enumTextArray->AppendL(new (ELeave) CAknEnumeratedText(0, resourceBuf.AllocL()));
		}

	TInt entryCount = entryList->Count();

	// Add all of the entries into the text arrays.

	for (TInt i = 0; i < entryCount; i++)
		{
		formatBuf.Copy((*entryList)[i].iName.Left((*entryList)[i].iName.Length() - 7));
		poppedUpTextArray->AppendL(formatBuf.AllocL());
		
		CAknEnumeratedText* enumText = new (ELeave)
			CAknEnumeratedText(i, formatBuf.AllocL());

		CleanupStack::PushL(enumText);

		enumTextArray->AppendL(enumText);
		}

	aSettingItem->SetEnumeratedTextArrays(enumTextArray, poppedUpTextArray);

	CleanupStack::Pop(entryCount + 2); // enumText, poppedUpTextArray, enumTextArray

	// Make sure that a valid entry is selected.

	if (entryCount <= iAppUi->Settings().iSkinFileIndex)
		iAppUi->Settings().iSkinFileIndex = 0;
	if (entryCount <= iAppUi->Settings().iUninstallSkinIndex)
		iAppUi->Settings().iUninstallSkinIndex = 0;

	aSettingItem->HandleTextArrayUpdateL();

	CleanupStack::PopAndDestroy(); // entryList
	}

void CBemusedDisplaySettingsContainer::GetInboxSkinListL(CAknEnumeratedTextPopupSettingItem* aSettingItem)
	{
	TBuf<256> resourceBuf;
	CEikonEnv::Static()->ReadResource(resourceBuf, R_SELECT_SKIN_MSG);

	// This function gets a list of skin files in the Inbox.

	iInboxSkinBitmapFileNames->Reset();
	iInboxSkinDescriptionFileNames->Reset();

	// Create text arrays and append a "select" item to both of them.

	CArrayPtrFlat<CAknEnumeratedText>* enumTextArray =
		new (ELeave) CArrayPtrFlat<CAknEnumeratedText>(4);
	CleanupStack::PushL(enumTextArray);

	CArrayPtrFlat<HBufC>* poppedUpTextArray = new (ELeave) CArrayPtrFlat<HBufC>(4);
	CleanupStack::PushL(poppedUpTextArray);

	poppedUpTextArray->AppendL(resourceBuf.AllocL());
	enumTextArray->AppendL(new (ELeave) CAknEnumeratedText(0, resourceBuf.AllocL()));

	// Find the Inbox storage drive
	TInt msgStoreDrv = MessageServer::CurrentDriveL(CEikonEnv::Static()->FsSession());
	TBuf<256> inboxPathAndDrive;
	inboxPathAndDrive.Format(_L("%C:%S"), 'A'+(msgStoreDrv-EDriveA), &KInboxPath);

	// Specify the Inbox directory.
	TBuf<256> totalPath;
	totalPath.Format(_L("%S\\%S"), &inboxPathAndDrive, &KMatchAll);

	// TEntryArrays are very large objects, so we allocate them on the heap.
	TEntryArray* entryArray1 = new (ELeave) TEntryArray;
	CleanupStack::PushL(entryArray1);

	TEntryArray* entryArray2 = new (ELeave) TEntryArray;
	CleanupStack::PushL(entryArray2);

	TEntryArray* entryArray3 = new (ELeave) TEntryArray;
	CleanupStack::PushL(entryArray3);

	RDir dir;

	// Open the Inbox directory.
	User::LeaveIfError(dir.Open(CEikonEnv::Static()->FsSession(), totalPath, KEntryAttDir));
	dir.Read(*entryArray1);
	dir.Close();

	TBuf<256> formatBuf;

	TInt totalFileCount = 0;

	// entryArray now contains the names of all the directories one level down from
	// the Inbox, ie. "0", "1", etc.
	for (TInt i = 0; i < entryArray1->Count(); i++)
		{
		// Specify each directory one level down from the Inbox in turn.
		totalPath.Format(_L("%S\\%S\\%S"), &inboxPathAndDrive, &(*entryArray1)[i].iName,
			&KMatchAll);

		User::LeaveIfError(dir.Open(CEikonEnv::Static()->FsSession(), totalPath, KEntryAttDir));
		dir.Read(*entryArray2);
		dir.Close();

		// entryArray2 now contains the names of all the directories a further level
		// down, ie. "001000a0_F" etc.

		for (TInt j = 0; j < entryArray2->Count(); j++)
			{
			// Specify each of the entries two levels down from the Inbox in turn.
			if (iAppUi->Settings().iDisplayType == TBemusedSettings::EInWindow)
				{
				totalPath.Format(_L("%S\\%S\\%S\\%S"), &inboxPathAndDrive, &(*entryArray1)[i].iName,
					&(*entryArray2)[j].iName, &KMatchInWindowSkins);
				}
			else
				{
				totalPath.Format(_L("%S\\%S\\%S\\%S"), &inboxPathAndDrive, &(*entryArray1)[i].iName,
					&(*entryArray2)[j].iName, &KMatchFullScreenSkins);
				}

			TInt rc = dir.Open(CEikonEnv::Static()->FsSession(), totalPath, KEntryAttDir);

			// If we can't open this directory, don't worry - just go on to the next one.
			if (rc != KErrNone)
				continue;

			dir.Read(*entryArray3);
			dir.Close();

			// entryArray3 now contains information about all the files in this
			// directory, hooray!

			for (TInt k = 0; k < entryArray3->Count(); k++)
				{
				totalPath.Format(_L("%S\\%S\\%S\\%S"), &inboxPathAndDrive, &(*entryArray1)[i].iName,
					&(*entryArray2)[j].iName, &(*entryArray3)[k].iName);

				// If we have a skin bitmap file, we need to append it to the text arrays for
				// display. If we have a skin description file, we append it to the list of
				// description files.
				if (!(*entryArray3)[k].iName.Right(3).CompareF(_L("png")))
					{
//					CEikonEnv::Static()->InfoWinL(_L("Found png file"), totalPath);

					iInboxSkinBitmapFileNames->AppendL(totalPath);

					formatBuf.Copy((*entryArray3)[k].iName.Left((*entryArray3)[k].iName.Length() - 7));
					poppedUpTextArray->AppendL(formatBuf.AllocL());
					
					CAknEnumeratedText* enumText = new (ELeave)
						CAknEnumeratedText(totalFileCount + 1, formatBuf.AllocL());

					CleanupStack::PushL(enumText);
					enumTextArray->AppendL(enumText);

					totalFileCount++;
					}
				else if (!(*entryArray3)[k].iName.Right(3).CompareF(_L("txt")))
					{
//					CEikonEnv::Static()->InfoWinL(_L("Found txt file"), totalPath);
					iInboxSkinDescriptionFileNames->AppendL(totalPath);
					}
				}		
			}
		}

	// Set the text arrays and update the list.

	aSettingItem->SetEnumeratedTextArrays(enumTextArray, poppedUpTextArray);

	if (totalFileCount <= iAppUi->Settings().iInstallSkinIndex)
		iAppUi->Settings().iInstallSkinIndex = 0;

	aSettingItem->HandleTextArrayUpdateL();
	CleanupStack::Pop(totalFileCount);

	CleanupStack::PopAndDestroy(3); // entryArray1, entryArray2, entryArray3

	CleanupStack::Pop(2); // poppedUpTextArray, enumTextArray
	}

void CBemusedDisplaySettingsContainer::ConstructL(CBemusedAppUi* aAppUi,
												  CBemusedDisplaySettingsView* aView,
												  const TRect& aRect)
    {
	iAppUi = aAppUi;
	iView = aView;

	iInboxSkinBitmapFileNames = new (ELeave) CDesC16ArrayFlat(2);
	iInboxSkinDescriptionFileNames = new (ELeave) CDesC16ArrayFlat(2);

    CreateWindowL();

    SetRect(aRect);
    ActivateL();

	iListBox = new (ELeave) CBemusedDisplaySettingItemList(iAppUi->Settings());
    iListBox->SetMopParent(this);
    iListBox->ConstructFromResourceL(R_BEMUSED_DISPLAY_SETTING_ITEM_LIST);

	// We have to call this after the list box has been constructed from the resource.
	GetSkinListL((CAknEnumeratedTextPopupSettingItem*)
		(*iListBox->SettingItemArray())[KSkinSettingIndex], EFalse);
	GetSkinListL((CAknEnumeratedTextPopupSettingItem*)
		(*iListBox->SettingItemArray())[KUninstallSkinSettingIndex], ETrue);

	GetInboxSkinListL((CAknEnumeratedTextPopupSettingItem*)
		(*iListBox->SettingItemArray())[KInstallSkinSettingIndex]);

	// We have to call this after the skin list has been obtained.
	iListBox->LoadSettingsL();

    iListBox->SetRect(aRect);
    iListBox->ActivateL();
    }

CBemusedDisplaySettingsContainer::~CBemusedDisplaySettingsContainer()
    {
	if (iListBox)
		{
		TRAPD(ignore, iListBox->StoreSettingsL());
		}

	delete iInboxSkinBitmapFileNames;
	delete iInboxSkinDescriptionFileNames;
	delete iListBox;
    }

void CBemusedDisplaySettingsContainer::SizeChanged()
    {
	if (iListBox)
		iListBox->SetRect(Rect());
    }

TInt CBemusedDisplaySettingsContainer::CountComponentControls() const
    {
	if (iListBox)
		return 1;
	else
		return 0;
    }

CCoeControl* CBemusedDisplaySettingsContainer::ComponentControl(TInt aIndex) const
    {
    switch (aIndex)
        {
        case 0:
			return iListBox;
        default:
            return NULL;
        }
    }

void CBemusedDisplaySettingsContainer::Draw(const TRect& aRect) const
    {
    CWindowGc& gc = SystemGc();

    gc.SetPenStyle(CGraphicsContext::ENullPen);
    gc.SetBrushColor(KRgbGray);
    gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
    gc.DrawRect(aRect);
    }

void CBemusedDisplaySettingsContainer::HandleControlEventL(CCoeControl* /*aControl*/,
													TCoeEvent /*aEventType*/)
    {
    }

TKeyResponse CBemusedDisplaySettingsContainer::OfferKeyEventL(const TKeyEvent& aKeyEvent,
													   TEventCode aType)
    {
	TBuf<256> resourceBuf;
	CEikonEnv::Static()->ReadResource(resourceBuf, R_SELECT_SKIN_MSG);

	if (iListBox)
		{
		if ((aKeyEvent.iCode == EKeyUpArrow) || (aKeyEvent.iCode == EKeyDownArrow) ||
			(aKeyEvent.iCode == EKeyDevice3))
			{
			// Pass joystick up, down and pressed events to the list box.
			TKeyResponse response = iListBox->OfferKeyEventL(aKeyEvent, aType);

			if ((response == EKeyWasConsumed) && (aKeyEvent.iCode == EKeyDevice3))
				{
				// There are three special cases: when changing the screen size, installing or
				// uninstalling a skin, we have to do some special processing.

				if (iListBox->ListBox()->CurrentItemIndex() == 0)
					{
					// We've changed the screen size, so update the settings list.
					iListBox->StoreSettingsL();

					GetSkinListL((CAknEnumeratedTextPopupSettingItem*)
						(*iListBox->SettingItemArray())[KSkinSettingIndex], EFalse);
					GetSkinListL((CAknEnumeratedTextPopupSettingItem*)
						(*iListBox->SettingItemArray())[KUninstallSkinSettingIndex], ETrue);

					GetInboxSkinListL((CAknEnumeratedTextPopupSettingItem*)
						(*iListBox->SettingItemArray())[KInstallSkinSettingIndex]);

					iListBox->LoadSettingsL();
					}

				else if (iListBox->ListBox()->CurrentItemIndex() == KInstallSkinSettingIndex)
					{
					// We've selected to install a skin. We need to find it, install it, and update
					// the settings list.

					iListBox->StoreSettingsL();

					CAknEnumeratedTextPopupSettingItem* settingItem =
						(CAknEnumeratedTextPopupSettingItem*)
							(*iListBox->SettingItemArray())[KInstallSkinSettingIndex];

					const TDesC& settingText = settingItem->SettingTextL();
					if (settingText != resourceBuf)
						{
						TBuf<256> formatBuf;

						for (TInt i = 0; i < iInboxSkinBitmapFileNames->Count(); i++)
							{
							TInt lastBackslash = (*iInboxSkinBitmapFileNames)[i].LocateReverse('\\');
							formatBuf.Copy((*iInboxSkinBitmapFileNames)[i].Mid(lastBackslash + 1,
								(*iInboxSkinBitmapFileNames)[i].Length() - lastBackslash - 8));

							if (!formatBuf.CompareF(settingText))
								{
								for (TInt j = 0; j < iInboxSkinDescriptionFileNames->Count(); j++)
									{
									lastBackslash =
										(*iInboxSkinDescriptionFileNames)[j].LocateReverse('\\');
									formatBuf.Copy((*iInboxSkinDescriptionFileNames)[j].
										Mid(lastBackslash + 1,
											(*iInboxSkinDescriptionFileNames)[j].Length() -
											lastBackslash - 8));

									if (!formatBuf.CompareF(settingText))
										{
										InstallSkin((*iInboxSkinBitmapFileNames)[i],
											(*iInboxSkinDescriptionFileNames)[j]);
										iListBox->ListBox()->SetCurrentItemIndexAndDraw(1);

										GetSkinListL((CAknEnumeratedTextPopupSettingItem*)
											(*iListBox->SettingItemArray())[KSkinSettingIndex], EFalse);
										GetSkinListL((CAknEnumeratedTextPopupSettingItem*)
											(*iListBox->SettingItemArray())[KUninstallSkinSettingIndex], ETrue);

										iListBox->LoadSettingsL();

										return response;
										}
									}
								}
							}
						}
					}

				else if (iListBox->ListBox()->CurrentItemIndex() == KUninstallSkinSettingIndex)
					{
					// We've selected to uninstall a skin. We need to find it, delete it, and
					// update the settings list.

					iListBox->StoreSettingsL();

					CAknEnumeratedTextPopupSettingItem* settingItem =
						(CAknEnumeratedTextPopupSettingItem*)
							(*iListBox->SettingItemArray())[KUninstallSkinSettingIndex];

					const TDesC& settingText = settingItem->SettingTextL();
					if (settingText != resourceBuf)
						{
						TBuf<256> resourceBuf;
						iAppUi->CoeEnv()->ReadResource(resourceBuf,
							R_UNINSTALL_SKIN_QUERY);

						TBuf<256> formatBuf;
						formatBuf.Format(resourceBuf, &settingText);
						
						if (CEikonEnv::Static()->QueryWinL(formatBuf, KNullDesC))
							{
							if (iAppUi->Settings().iDisplayType == TBemusedSettings::EFullScreen)
								{
								formatBuf.Format(_L("%S\\%S_fs.png"), &iAppUi->SkinPathL(), &settingText);
								CEikonEnv::Static()->FsSession().Delete(formatBuf);
								formatBuf.Format(_L("%S\\%S_fs.txt"), &iAppUi->SkinPathL(), &settingText);
								CEikonEnv::Static()->FsSession().Delete(formatBuf);
								}
							else
								{
								formatBuf.Format(_L("%S\\%S_iw.png"), &iAppUi->SkinPathL(), &settingText);
								CEikonEnv::Static()->FsSession().Delete(formatBuf);
								formatBuf.Format(_L("%S\\%S_iw.txt"), &iAppUi->SkinPathL(), &settingText);
								CEikonEnv::Static()->FsSession().Delete(formatBuf);
								}

							GetSkinListL((CAknEnumeratedTextPopupSettingItem*)
								(*iListBox->SettingItemArray())[KSkinSettingIndex], EFalse);
							GetSkinListL((CAknEnumeratedTextPopupSettingItem*)
								(*iListBox->SettingItemArray())[KUninstallSkinSettingIndex], ETrue);
							}
						else
							{
							iAppUi->Settings().iUninstallSkinIndex = 0;
							}

						iListBox->LoadSettingsL();
						}
					}
				}
			return response;
			}
		}
	return EKeyWasNotConsumed;
	}

void CBemusedDisplaySettingsContainer::InstallSkin(const TDesC& aBitmapFileName,
											const TDesC& aDescriptionFileName)
	{
	// This function copies the skin bitmap and description files from the Inbox into the Bemused
	// skins directory.

	TInt lastBackslash = aDescriptionFileName.LocateReverse('\\');
	TPtrC filePath = aDescriptionFileName.Left(lastBackslash);
	TPtrC nameWithoutExtension = aDescriptionFileName.Mid(lastBackslash + 1,
		aDescriptionFileName.Length() - lastBackslash - 5);

	TBuf<256> sourceBuf;
	TBuf<256> destBuf;
	TBuf<256> resourceBuf;

	sourceBuf.Format(_L("%S\\%S.txt"), &filePath, &nameWithoutExtension);
	destBuf.Format(_L("%S\\%S.txt"), &iAppUi->SkinPathL(), &nameWithoutExtension);

	TInt ret = EikFileUtils::CopyFile(sourceBuf, destBuf);
	if (ret != KErrNone)
		{
		iAppUi->CoeEnv()->ReadResource(resourceBuf, R_COPY_SKIN_DESCRIPTION_ERROR);
		CEikonEnv::Static()->InfoWinL(resourceBuf, KNullDesC);
		return;
		}

	lastBackslash = aBitmapFileName.LocateReverse('\\');
	filePath.Set(aBitmapFileName.Left(lastBackslash));
	nameWithoutExtension.Set(aBitmapFileName.Mid(lastBackslash + 1,
		aBitmapFileName.Length() - lastBackslash - 5));

	sourceBuf.Format(_L("%S\\%S.png"), &filePath, &nameWithoutExtension);
	destBuf.Format(_L("%S\\%S.png"), &iAppUi->SkinPathL(), &nameWithoutExtension);

	ret = EikFileUtils::CopyFile(sourceBuf, destBuf);
	if (ret != KErrNone)
		{
		iAppUi->CoeEnv()->ReadResource(resourceBuf, R_COPY_SKIN_BITMAP_ERROR);
		CEikonEnv::Static()->InfoWinL(resourceBuf, KNullDesC);

		// delete the skin description file
		destBuf.Format(_L("%S\\%S.txt"), &iAppUi->SkinPathL(), &nameWithoutExtension);
		EikFileUtils::DeleteFile(destBuf);
		return;
		}

	iAppUi->CoeEnv()->ReadResource(resourceBuf, R_SKIN_INSTALLED_MSG);
	CEikonEnv::Static()->InfoWinL(resourceBuf, KNullDesC);
	}

// End of File  
