/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#include <eikappui.h>
#include <qiktabscreen.h>
#include <barsread.h>
#include <eiktxlbm.h>
#include <eikmenub.h>

#include <Bemused.rsg>
#include "Bemused.hrh"
#include "BemusedMainView.h"
#include "BemusedAppUi.h"
#include "BemusedBrowseContainer.h" 
#include "BemusedControlContainer.h" 
#include "BemusedApp.h"
#include "BemusedListBox.h"
#include "BemusedFindSongDialog.h"
#include "BemusedFlipClosedView.h"

void CBemusedMainView::ConstructL(CBemusedAppUi* aAppUi)
    {
	iAppUi = aAppUi;

	iTextForegroundColour = KRgbBlack;
	iTextBackgroundColour = KRgbWhite;
	iHighlightForegroundColour = KRgbBlack;
	iHighlightBackgroundColour = TRgb(0x80, 0x80, 0xFF);

	CreateWindowL();
	SetRect(aAppUi->ClientRect());

	// Create a tab screen object on the heap. 
	iTabScreen = new (ELeave) CQikTabScreen;
	iTabScreen->SetContainerWindowL(*this);

	// Create a resource reader for reading in the specified resource from the resource file.
    TResourceReader resourceReader;
	iCoeEnv->CreateResourceReaderLC(resourceReader, R_BEMUSED_TAB_SCREEN);

	// Complete construction of the newly-allocated tab screen object from the QIK_TABSCREEN resource. 
	iTabScreen->ConstructFromResourceL(resourceReader);

	// Pop and destroy resourceReader from cleanup stack.
	CleanupStack::PopAndDestroy();

	// Set tab screen control's extent.
	iTabScreen->SetExtentToAllAvailableSpaceL();

	// Create the first page.

	iBrowsePage = (CQikTabScreenPage*)iTabScreen->PageControl(0); 
	iBrowsePage->ConstructL(*iTabScreen);

    iBrowseContainer = new (ELeave) CBemusedBrowseContainer;
	iBrowsePage->AddControlL(iBrowseContainer); // takes ownership

	iBrowseContainer->SetColours(iTextForegroundColour, iTextBackgroundColour,
		iHighlightForegroundColour, iHighlightBackgroundColour);

	TRect boundingRect(iAppUi->ClientRect().iTl,
		iAppUi->ClientRect().iTl + iBrowsePage->Rect().iBr);

	iBrowseContainer->ConstructL(iAppUi, boundingRect);

	// Create the second page.

	iControlPage = (CQikTabScreenPage*)iTabScreen->PageControl(1);
	iControlPage->ConstructL(*iTabScreen);

    iControlContainer = new (ELeave) CBemusedControlContainer;

	iControlPage->AddControlL(iControlContainer); // takes ownership

	iControlContainer->ConstructL(iAppUi, this, TRect(iAppUi->ClientRect().iTl,
		iAppUi->ClientRect().iTl + TSize(KFullScreenWidth, KFullScreenHeight)));

	// Create the third page.
	iPlaylistPage = (CQikTabScreenPage*)iTabScreen->PageControl(2);
	iPlaylistPage->ConstructL(*iTabScreen);
	CreatePlaylistBoxL();

	iSkinHandler = CBemusedSkinHandler::NewL(iAppUi, iSkin);

	iTabScreen->SetObserver(this);

	iControlContainer->MakeVisible(EFalse);
	iBrowsePage->SetFocus(ETrue);

	MakeVisible(EFalse);
	ActivateL();
    }

CBemusedMainView::~CBemusedMainView()
    {
	delete iPlaylistItems;
	delete iSkinHandler;
    delete iTabScreen;
    }

TUid CBemusedMainView::Id() const
    {
    return KMainViewId;
    }

void CBemusedMainView::HandleCommandL(TInt aCommand)
    {
    }

void CBemusedMainView::HandleClientRectChange()
    {
	if (iTabScreen)
		iTabScreen->SetRect(Rect());
    }

void CBemusedMainView::ReadSkinDescriptionL(const TDesC& aFileName)
	{
	// This function reads the bits of the skin description which this
	// view is interested in.

	// Set default values in case we can't open the skin.
	iTextForegroundColour = KRgbBlack;
	iTextBackgroundColour = KRgbWhite;
	iHighlightForegroundColour = KRgbBlack;
	iHighlightBackgroundColour = TRgb(0x80, 0x80, 0xFF);

	RFile file;
	User::LeaveIfError(file.Open(CEikonEnv::Static()->FsSession(),
		aFileName, EFileShareAny | EFileRead));
	CleanupClosePushL(file);

	TInt fileSize;
	User::LeaveIfError(file.Size(fileSize));

	HBufC8* fileBuf = HBufC8::NewLC(fileSize);
	TPtr8 fileBufPtr = fileBuf->Des();

	User::LeaveIfError(file.Read(fileBufPtr));

	TLex8 lex(*fileBuf);
	TPtrC8 ptr(lex.NextToken());

	while (ptr != KNullDesC8)
		{
		lex.SkipSpace();

		if (!ptr.CompareF(_L8("textforegroundcolour:")))
			ReadSkinColour(lex, iTextForegroundColour);
		else if (!ptr.CompareF(_L8("textbackgroundcolour:")))
			ReadSkinColour(lex, iTextBackgroundColour);
		else if (!ptr.CompareF(_L8("highlightforegroundcolour:")))
			ReadSkinColour(lex, iHighlightForegroundColour);
		else if (!ptr.CompareF(_L8("highlightbackgroundcolour:")))
			ReadSkinColour(lex, iHighlightBackgroundColour);
		else
			lex.SkipSpace();

		ptr.Set(lex.NextToken());
		}

	CleanupStack::PopAndDestroy(2); // file, fileBuf

	iBrowseContainer->SetColours(iTextForegroundColour, iTextBackgroundColour,
			iHighlightForegroundColour, iHighlightBackgroundColour);

	CBemusedListItemDrawer* drawer = (CBemusedListItemDrawer*) iPlaylistBox->View()->ItemDrawer();

	drawer->SetColours(iTextForegroundColour, iTextBackgroundColour, iHighlightForegroundColour,
		iHighlightBackgroundColour);

	// Set the colour to be used in areas where there aren't any items in the list.
	iPlaylistBox->View()->SetBackColor(iTextBackgroundColour);
	}

void CBemusedMainView::ReadSkinColour(TLex8& aLex, TRgb& aColour)
	{
	TInt colour;

	aLex.Val(colour);
	aColour.SetRed(colour);
	aLex.SkipSpace();

	aLex.Val(colour);
	aColour.SetGreen(colour);
	aLex.SkipSpace();

	aLex.Val(colour);
	aColour.SetBlue(colour);
	aLex.SkipSpace();
	}

CBemusedBrowseContainer* CBemusedMainView::BrowseContainer()
	{
	if (iTabScreen->CurrentTabIndex() == 0)
		return iBrowseContainer;

	return NULL;
	}

CBemusedControlContainer* CBemusedMainView::ControlContainer()
	{
	if (iTabScreen->CurrentTabIndex() == 1)
		return iControlContainer;

	return NULL;
	}

void CBemusedMainView::SetListLoaded(TBool aListLoaded)
	{
//	if (iBrowseContainer && !iListLoaded && aListLoaded)
//		{
//		TBuf<256> resourceBuf;
//		((CBemusedAppUi*)AppUi())->CoeEnv()->ReadResource(resourceBuf, R_EMPTY_SONG_LIST);
//		iBrowseContainer->SetListEmptyStringL(resourceBuf);
//		}

	iListLoaded = aListLoaded;
	}

TVwsViewId CBemusedMainView::ViewId() const
	{
	return TVwsViewId(KUidBemused, Id());
	}

void CBemusedMainView::ViewActivatedL(const TVwsViewId& aPrevViewId,
		TUid aCustomMessageId, const TDesC8& aCustomMessage)
	{
	TPixelsAndRotation sizeAndRotation;
	CEikonEnv::Static()->ScreenDevice()->GetDefaultScreenSizeAndRotation(sizeAndRotation);
	CEikonEnv::Static()->ScreenDevice()->SetScreenSizeAndRotation(sizeAndRotation);
	CEikonEnv::Static()->AppUiFactory()->MenuBar()->MakeVisible(ETrue);

	if (iActivateCount == 0)
		MakeVisible(ETrue);
	iActivateCount++;

	iAppUi->SetActiveView(KMainViewId);
	}

void CBemusedMainView::ViewDeactivated()
	{
	iActivateCount--;
	if (iActivateCount == 0)
		MakeVisible(EFalse);
	}

void CBemusedMainView::HandleControlEventL(CCoeControl* aControl, TCoeEvent aEventType)
	{
	if ((aControl == iTabScreen) && (aEventType == EEventStateChanged))
		{
		switch (iTabScreen->CurrentTabIndex())
			{
		case 0:
			iControlContainer->MakeVisible(EFalse);
			iPlaylistBox->MakeVisible(EFalse);
			iBrowseContainer->MakeVisible(ETrue);

			CEikonEnv::Static()->AppUiFactory()->
				MenuBar()->ChangeMenuBarL(R_BEMUSED_HOTKEYS, R_BEMUSED_BROWSE_MENUBAR, EFalse);
			break;

		case 1:
			iBrowseContainer->MakeVisible(EFalse);
			iPlaylistBox->MakeVisible(EFalse);
			iControlContainer->MakeVisible(ETrue);

			CEikonEnv::Static()->AppUiFactory()->
				MenuBar()->ChangeMenuBarL(R_BEMUSED_HOTKEYS, R_BEMUSED_CONTROL_MENUBAR, EFalse);
			break;

		case 2:
			iAppUi->GetPlaylist();
			iBrowseContainer->MakeVisible(EFalse);
			iControlContainer->MakeVisible(EFalse);
			iPlaylistBox->MakeVisible(ETrue);

			CEikonEnv::Static()->AppUiFactory()->
				MenuBar()->ChangeMenuBarL(R_BEMUSED_HOTKEYS, R_BEMUSED_PLAYLIST_MENUBAR, EFalse);
			break;

		default:
			break;
			}
		DrawNow();
		}
	}

void CBemusedMainView::CreatePlaylistBoxL()
	{
	const CFont* font;

	switch (iAppUi->Settings().iSongBrowserFont)
		{
	case TBemusedSettings::EHuge:
		font = CEikonEnv::Static()->TitleFont();
		break;
	case TBemusedSettings::ELarge:
		font = CEikonEnv::Static()->LegendFont();
		break;
	case TBemusedSettings::EMedium:
		font = CEikonEnv::Static()->NormalFont();
		break;
	case TBemusedSettings::ESmall:
	default:
		font = CEikonEnv::Static()->DenseFont();
		break;
		}

	iPlaylistItems = new (ELeave) CDesC16ArrayFlat(4);

	iPlaylistBox = new (ELeave) CBemusedListBox(font);

	iPlaylistPage->AddControlL(iPlaylistBox); // takes ownership

	iPlaylistBox->ConstructL(NULL, CEikListBox::EPaintedSelection);

	iPlaylistBox->Model()->SetOwnershipType(ELbmDoesNotOwnItemArray);
	iPlaylistBox->Model()->SetItemTextArray(iPlaylistItems);

	// Set the colour to be used in areas where there aren't any items in the list.
	iPlaylistBox->View()->SetBackColor(iTextBackgroundColour);
	iPlaylistBox->View()->ItemDrawer()->SetBackColor(iTextBackgroundColour);

	// Set the item height to a sensible value that can be used for all fonts.
	iPlaylistBox->SetItemHeightL(KItemHeight);

	// Create the scrolling indicators at the bottom of the screen.
	iPlaylistScrollBarFrame = iPlaylistBox->CreateScrollBarFrameL(ETrue);
	iPlaylistBox->ScrollBarFrame()->SetScrollBarVisibilityL(
			CEikScrollBarFrame::EAuto, CEikScrollBarFrame::EAuto);

	TRect boundingRect(iAppUi->ClientRect().iTl,
		iAppUi->ClientRect().iTl + iBrowsePage->Rect().iBr);

	iPlaylistBox->SetRect(boundingRect);

	CBemusedListItemDrawer* drawer = (CBemusedListItemDrawer*) iPlaylistBox->View()->ItemDrawer();

	drawer->SetColours(iTextForegroundColour, iTextBackgroundColour, iHighlightForegroundColour,
		iHighlightBackgroundColour);

	iPlaylistBox->SetListBoxObserver(this);

	iPlaylistBox->MakeVisible(EFalse);
	}

void CBemusedMainView::RefreshPlaylistL(const RPointerArray<HBufC8>& aPlaylist, TInt aIndex)
	{
	iPlaylistItems->Reset();

	for (TInt i = 0; i < aPlaylist.Count(); i++)
		{
		HBufC* fileBuf = HBufC::NewLC(aPlaylist[i]->Length());
		fileBuf->Des().Copy(*aPlaylist[i]);
		iPlaylistItems->AppendL(*fileBuf);
		CleanupStack::PopAndDestroy();
		}

	iPlaylistBox->Model()->SetItemTextArray(iPlaylistItems);
	iPlaylistBox->HandleItemAdditionL();

	if ((aIndex >= 0) && (aIndex <= aPlaylist.Count()))
		{
		iPlaylistBox->SetCurrentItemIndex(aIndex);
		iPlaylistBox->ScrollToMakeItemVisible(aIndex);
		}

	iPlaylistBox->DrawNow();
	}

void CBemusedMainView::SelectPreviousFileInPlaylist()
	{
	TInt currentIndex = iPlaylistBox->CurrentItemIndex();

	if (currentIndex > 0)
		{
		iPlaylistBox->SetCurrentItemIndex(currentIndex - 1);
		iPlaylistBox->ScrollToMakeItemVisible(currentIndex - 1);
		iPlaylistBox->DrawDeferred();
		}
	}

void CBemusedMainView::SelectNextFileInPlaylist()
	{
	TInt currentIndex = iPlaylistBox->CurrentItemIndex();

	if (currentIndex < (iPlaylistItems->Count()-1))
		{
		iPlaylistBox->SetCurrentItemIndex(currentIndex + 1);
		iPlaylistBox->ScrollToMakeItemVisible(currentIndex + 1);
		iPlaylistBox->DrawDeferred();
		}
	}

void CBemusedMainView::PlaylistFileSelected()
	{
	iAppUi->SelectInPlaylist(iPlaylistBox->CurrentItemIndex());
	}

void CBemusedMainView::FindSongInPlaylistL(TBool aFindNext)
	{
	TFileName lastSearchedSong = iLastSearchedSong;
	CBemusedFindSongDialog* dialog = NULL;

	// Create a dialog to ask the user what to find, if we don't want
	// to just find the next occurrence.
	if (aFindNext)
		{
		if (iLastSearchedSong == KNullDesC)
			return;
		}
	else
		{
		dialog = new (ELeave) CBemusedFindSongDialog(iLastSearchedSong);
		CleanupStack::PushL(dialog);
		}

	iResultsFound = 0;

	if (aFindNext || dialog->ExecuteLD(R_FIND_DIALOG))
		{
		// If the user wants to search for something else, don't skip
		// any of the results.
		if (iLastSearchedSong != lastSearchedSong)
			iFindResultsToSkip = 0;

		TInt count = iPlaylistBox->Model()->ItemTextArray()->MdcaCount();
		TInt resultsFound = 0;

		for (TInt i = 0; i < count; i++)
			{
			if ((*iPlaylistItems)[i].FindF(iLastSearchedSong) != KErrNotFound)
				{
				resultsFound++;

				if (resultsFound > iFindResultsToSkip)
					{
					((CBemusedListItemDrawer*) iPlaylistBox->View()->ItemDrawer())->ResetScrolling();
					iPlaylistBox->SetCurrentItemIndex(i);
					iPlaylistBox->ScrollToMakeItemVisible(i);
					iPlaylistBox->DrawNow();

					// If the user searches again, we'll skip this result.
					iFindResultsToSkip++;

					if (!aFindNext)
						CleanupStack::Pop(); // dialog
					return;
					}
				}
			}

		lastSearchedSong.Format(_L("Couldn't find \"%S\""), &iLastSearchedSong);
		CEikonEnv::Static()->InfoWinL(KNullDesC, lastSearchedSong);
		iFindResultsToSkip = 0;
		}

	if (!aFindNext)
		CleanupStack::Pop(); // dialog
	}

TVwsViewIdAndMessage CBemusedMainView::ViewScreenDeviceChangedL()
	{
	return TVwsViewIdAndMessage(TVwsViewId(KUidBemused, KFlipClosedViewId));
	}

TBool CBemusedMainView::ViewScreenModeCompatible(TInt aScreenMode)
	{
	return (aScreenMode == EScreenModeFlipOpen);
	}

void CBemusedMainView::HandleListBoxEventL(CEikListBox* aListBox, TListBoxEvent aEventType)
	{
	if (aEventType == EEventItemClicked)
		iAppUi->SelectInPlaylist(aListBox->CurrentItemIndex());
	}

TInt CBemusedMainView::CountComponentControls() const
	{
	if (iTabScreen)
		return 1;

	return 0;
	}

CCoeControl* CBemusedMainView::ComponentControl(TInt aIndex) const
	{
	if (aIndex == 0)
		return iTabScreen;

	return NULL;
	}

TBemusedSkin& CBemusedMainView::Skin()
	{
	return iSkin;
	}

CBemusedSkinHandler* CBemusedMainView::SkinHandler()
	{
	return iSkinHandler;
	}

TInt CBemusedMainView::TabIndex()
	{
	return iTabScreen->CurrentTabIndex();
	}

CQikTabScreen* CBemusedMainView::TabScreen()
	{
	return iTabScreen;
	}

// End of File

